#ifndef CHAT_H
#define CHAT_H

#include <qcolor.h>
#include <qdialog.h>
#include <qmap.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qrect.h>
#include <qsplitter.h>
#include <qvariant.h>
#include <qvaluelist.h>

#include "emoticons.h"
#include "misc.h"
#include "userbox.h"

class Chat;
class QHBox;
class ChatSlots;
/** \typedef QValueList<Chat*> ChatList */
typedef QValueList<Chat*> ChatList;

/**

	Klasa przechowujca informacje o wiadomoci, ktra ma si pojawi
	w oknie Chat.
	\class ChatMessage
	\brief Klasa przechowujca informacje o wiadomoci.
**/
class ChatMessage
{
	public:
	QString nick;/*!< nazwa uytkownika */
	QDateTime date; /*!< data otrzymania wiadomoci */
	QDateTime sdate; /*!< data wysania wiadomoci */
	QString unformattedMessage; /*!< niesformatowana wiadomo */
	bool isMyMessage; /*!< zmienna mowica czy wiadomoc zostaa 
				napisana przez nas */
	QColor backgroundColor; /*!< to wiadomoci */
	QColor textColor; /*!< kolor wiadomoci */
	
	
	QMap<QString, bool> attributes; /*!<
		Moe suy do przechowywania informacji o tym
		czy wiadomo bya potwierdzona, szyfrowana, ...
		W zamyle ma suy do okrelania czy jaki obrazek
		ma by dodawany do wiadomoci czy nie - jaki obrazek -
		ta informacja bdzie gdzie indziej
		*/
	
	//inne atrybuty?
	//QMap<QString, QString> stringAttributes;
	
	bool needsToBeFormatted;/*!< zmienna mowica czy wiadomoc powinna 
				  by sformatowana */
	
	QString message;/*!<
			Sformatowana wiadomo (razem z \<p\> lub \<table\>)
			**/


	/**
		\fn ChatMessage(const QString &nick, const QString &unformattedMessage, bool myMessage, QDateTime date, QDateTime sdate=QDateTime())
		Konstruktor ustawiajcy nick, niesformatowan wiadomo,
		czy wiadomo pochodzi od nas, dat wiadomoci,
		dat nadania wiadomoci przez serwer
		\param nick nazwa uytkownika
		\param unformattedMessage niesformatowana wiadomo
		\param myMessage zmienna mowica czy wiadomo pochodzi od nas
		\param date data otrzymania wiadomoci
		\param sdate data wysania wiadomoci
	**/
	ChatMessage(const QString &nick, const QString &unformattedMessage, bool myMessage, QDateTime date, QDateTime sdate=QDateTime());
	/**
		\fn ChatMessage(const QString &formattedMessage, const QColor &bgColor=Qt::white, const QColor &textColor=Qt::black)
		Konstruktor ustawiajcy sformatowan wiadomo,
		kolor ta i liter wiadomoci
		\param nick nazwa uytkownika
		\param unformattedMessage niesformatowana wiadomo
		\param myMessage zmienna mowica czy wiadomo pochodzi od nas
		\param date data otrzymania wiadomoci
		\param sdate data wysania wiadomoci
	**/
	ChatMessage(const QString &formattedMessage, const QColor &bgColor=Qt::white, const QColor &textColor=Qt::black);
};
/**
	Klasa pozwalajca zarzdza otwartymi oknami rozmw: otwiera,
	zamykac, szuka okna ze wgldu na list uytkownikw itp.
	\class ChatManager
	\brief Klasa zarzdzajca oknami Chat
**/
class ChatManager : public QObject
{
	Q_OBJECT

	private:
		ChatList Chats;/*!< lista okien*/
		
		static ChatSlots *chatslots;
		/**
		\fn int openPendingMsg(int index, ChatMessage &msg)
		Funkcja otwierajca zakolejkowan wiadomo
		\param index nr wiadomoci z kolejki
		\param msg wiadomo z kolejki
		\return zwracany jest nr okna w ktrym zostaa otwarta wiadomo
		**/
		int openPendingMsg(int index, ChatMessage &msg);
		
		/** 
		\struct ChatInfo
		Struktura przechowuje informacje o danym oknie
		**/
		struct ChatInfo
		{
			UinsList uins;/*!< lista uytkownikw 
					identyfikujca okno */
			QMap<QString, QVariant> map; /*!< parametry 
							danego okna */
		};
		QValueList<ChatInfo> addons; /*!< lista parametrw okien */

	public:	
		/**
		\fn ChatManager(QObject* parent=NULL, const char* name=NULL)
		Konstruktor tworzcy obiekt zarzdzajcy oknami
		\param parent rodzic okna
		\param name nazwa obiektu
		**/
		ChatManager(QObject* parent=NULL, const char* name=NULL);
		
		/**
			\fn ~ChatManager()
			Destruktor zamyka wszystkie otwarte okna
		**/
		
		~ChatManager();
		
		/**
			\fn const ChatList& chats() const
			Funkcja zwraca liste otwartych okien Chat
		**/

		/**
			\fn static void initModule()
			Rejestruje opcje modulu Chat w oknie konfiguracji
		**/
		static void initModule();
		static void closeModule();

		const ChatList& chats() const;
		
		/**
			\fn Chat* findChatByUins(const UinsList &uins) const
			Funkcja zwraca wskanik do okna z list 
			uytkownikw uins
			\param uins lista uytkownikw
			\return wskanik do okna jeli istnieje w przeciwnym
			 wypadku zwraca NULL
		**/
		Chat* findChatByUins(const UinsList &uins) const;
		
		/**
			\fn QVariant& getChatProperty(const UinsList &uins, const QString &name)
			Funkcja zwraca warto wasnoci "name" okna 
			okrelonego przez uins
			\param uins lista uytkownikw identyfikujca okno
			\param name nazwa wasnoci
			\return zwraca warto wasnoci jeli okrelone okno 
			istnieje,\n jeli nie to tworzy tak 
			wasno (ustawia na pust)
		**/
		QVariant& getChatProperty(const UinsList &uins, const QString &name);
		
	public slots:
	
		/**
			\fn void chatMsgReceived(UinsList senders,const QString& msg,time_t time,bool& grab)
			Slot informujcy o odebraniu wiadomoci
			\param senders lista uytkownikw identyfikujcych okno
			\param msg tre otrzymanej wiadomoci
			\param time czas otrzymania wiadomoci
			\param grab 
		**/
		void chatMsgReceived(UinsList senders,const QString& msg,time_t time,bool& grab);

		/**
			\fn int openChat(UinsList senders,time_t time=0)
			Funkcja otwiera nowe okno Chat z wymienionymi rozmowcami.
			\param senders lista uytkownikw identyfikujcych okno
			\param time parametr sluzy do sprawdzenia ile wiadomosci
			z historii ma sie pojawic w oknie.
			\return zwracany jest numer otwartego okna 
		**/
		int openChat(UinsList senders,time_t time=0);		
		
		/**
			\fn void openPendingMsgs(UinsList uins)
			Funkcja wpisuje zakolejkowane wiadomoci do okna
			z uytkownikami "uins"
			\param uins lista uytkownikw identyfikujcych okno
		**/
		void openPendingMsgs(UinsList uins);
		
		/**
			\fn void openPendingMsgs()
			Funkcja wpisuje wszystkie zakolejkowane wiadomoci 
			do odpowiednich okien
		**/
		void openPendingMsgs();
		
		//TODO: opisac funkcje sendMessage(..)
		/*
			Niebardzo rozumiem tej funkcji (czemu jest uin i uins)	
		*/
		void sendMessage(UinType uin,UinsList selected_uins);
		
		/**
			\fn void closeAllWindows()
			Funkcja zamyka wszystkie okna chat
		**/
		void closeAllWindows();
		
		/**
			\fn int registerChat(Chat* chat)
			Dodaje okno do menadera
			\param chat wskanik do okna ktore chcemy doda
			\return zwraca numer naszego okna po zarejestrowaniu
		**/
		int registerChat(Chat* chat);
		
		/**
			\fn void unregisterChat(Chat* chat)
			Funkcja wyrejestrowuje okno z managera \n
			Zapisuje wasnoci okna \n
			wysya sygna chatDestroying i chatDestroyed
			\param chat okno ktre bdzie wyrejestrowane
		**/
		void unregisterChat(Chat* chat);
		
		/**
			\fn void refreshTitles()
			Funkcja odwiea tytuy wszystkich okien
		**/
		void refreshTitles();
		
		/**
			\fn void refreshTitlesForUin(UinType uin)
			Funkcja odwiea tytuy okien ktre zawieraj uin
			\param uin nr uytkownika, ktrego 
			opis/status bdzie odwieany
		**/
		void refreshTitlesForUin(UinType uin);
		
		/**
			\fn void changeAppearance()
			Funkcja odwiea kolory i czcionki we wszystkich oknach
		**/
		void changeAppearance();
		
		/**
			\fn void setChatProperty(const UinsList &uins, const QString &name, const QVariant &value)
			Funkcja pozwala przypisa okrelonemu czatowi 
			(nawet jeeli on jeszcze nie istnieje) pewne wasnoci
			\param uins lista uytkownikw identyfikujcych okno
			\param name nazwa wasnoci
			\param value warto wasnoci
		**/
		void setChatProperty(const UinsList &uins, const QString &name, const QVariant &value);
		
	signals:
		
		/**
			\fn void chatCreated(const UinsList& senders)
		 	Sygna ten jest wysyany po utworzeniu nowego okna chat
			\param senders lista uin-w z ktrymi tworzymy nowy chat
		**/
		void chatCreated(const UinsList& senders);
		
		/**
			\fn void chatDestroying(const UinsList& senders)
		 	Sygna ten jest wysyany przed zamnkniciem okna chat
			\param senders lista uin-w ktre wystpuj w tym chacie
		**/
		void chatDestroying(const UinsList& senders);
		
		/**
			\fn void chatDestroyed(const UinsList& senders)
		 	Sygna ten jest wysyany po zamnkniciem okna chat
			\param senders lista uin-w ktre 
			wystpoway w tym chacie
		**/
		void chatDestroyed(const UinsList& senders);
		
		/**
			\fn void chatOpen(const UinsList& senders)
			Sygna ten jest wysyaniy podczas kadej prby 
			otwarcia nowego okna chat nawet jeli ju taki istnieje
			\param senders lista uin-w
		**/
		void chatOpen(const UinsList& senders);
};

/** 
	wskanik do obiektu ChatManager'a 
**/
extern ChatManager* chat_manager; 

class EmoticonSelector;
class ColorSelector;
/**
	\class CustomInput
	\brief Klasa umoliwiajca wpisywanie wiadomoci
**/
class CustomInput : public QMultiLineEdit
{
	Q_OBJECT

	protected:
		
		bool autosend_enabled;/*!< zmienna informujca czy wczone jest 
					automatyczne wysyanie */
		/**
			\fn virtual void keyPressEvent(QKeyEvent *e)
			Funkcja obsugujca nacinicie przycisku
			\param e wskanik do obiektu obsugujcego klawisze
		**/
		virtual void keyPressEvent(QKeyEvent *e);
		
		/**
			\fn virtual void keyReleaseEvent(QKeyEvent *e)
			Funkcja obsugujca zwolnienie przycisku
			\param e wskanik do obiektu obsugujcego klawisze
		**/
		virtual void keyReleaseEvent(QKeyEvent* e);

	public:
		
		/*! Typ wyliczeniowy mowicy o rodzaju pisanych znakw */
		enum
		{
			KEY_BOLD,/*!< pogrubienie */
			KEY_ITALIC,/*!< kursywa */
			KEY_UNDERLINE/*!< podkrelenie */
		};
		
		/**
			\fn CustomInput(QWidget* parent = 0, const char* name = 0)
			Konstruktor tworzcy obiekt 
			i ustawiajcy odpowiedni styl
			\param parent rodzic okna
			\param name nazwa obiektu
		**/
		CustomInput(QWidget* parent = 0, const char* name = 0);

	public slots:
		
		/**
			\fn void paste()
			Slot wkleja czysty tekst
		**/
		void paste();
		
		/**
			\fn void setAutosend(bool on)
			Ustawia/wycza automatyczne wysyanie wiadomoci
			\param on zmienna ustawiajca autosend
		**/
		void setAutosend(bool on);

	signals:
		
		/**
			\fn void sendMessage()
			Sygna zostaje wysany kiedy nacinito 
			klawisz wysania wiadomoci
		**/
		void sendMessage();
		
		/**
			\fn void specialKeyPressed(int key)
			Sygna zostaje wysany gdy nacinito specjalny 
			klawisz (skrt) odpowiadajcy np KEY_BOLD
			\param key warto z typu wyliczeniowego KEY_*
		**/
		void specialKeyPressed(int key);
		
		/**
			\fn void keyPressed(QKeyEvent* e, CustomInput* sender)
			Dowolny przycisk klawiatury zosta nacinity.
			Przekazany zostaje take obiekt, ktry wywoa akcj -
			 czyli this
			\param e wskanik do obiektu obsugujcego klawisze
			\param sender wskanik do obiektu, ktry
			wywoa ten sygna
		**/
		void keyPressed(QKeyEvent* e, CustomInput* sender);
		
		/**
			\fn void keyReleased(QKeyEvent* e, CustomInput* sender)
			Dowolny przycisk klawiatury zosta zwolniony.
			Przekazany zostaje take obiekt, ktry wywoa akcj -
			 czyli this.
			\param e wskanik do obiektu obsugujcego klawisze
			\param sender wskanik do obiektu, ktry
			wywoa ten sygna
		**/
		void keyReleased(QKeyEvent* e, CustomInput* sender);
};

/**
	Klasa wykorzystana do oddzielenia listy uytkownikw od historii
	i pola do wpisywania wiadomoci
	\class KaduSplitter
	\brief Klasa rozdzielajca kontrolki

**/
class KaduSplitter : public QSplitter
{
	protected:
		QValueList<KaduTextBrowser*> textbrowsers;
		void drawContents(QPainter* p);
		void childEvent(QChildEvent* c);

	public:
	/**
		Konstruktor tworzcy obiekt
		\fn KaduSplitter(QWidget* parent = 0, const char* name = 0)
		\param parent rodzic okna
		\param name nazwa obiektu

	**/
		KaduSplitter(QWidget* parent = 0, const char* name = 0);
	
	/**
		Konstruktor tworzcy obiekt o danej orientacji
		\fn KaduSplitter(Orientation o, QWidget* parent = 0, const char* name = 0)
		\param parent rodzic okna
		\param name nazwa obiektu
		\param o orientacja separatora
	**/
		KaduSplitter(Orientation o, QWidget* parent = 0, const char* name = 0);
};

class QMimeSourceFactory;

/**
	Klasa tworzca okno rozmowy, rejestrujca przyciski,
	formatujca wiadomo itp.
	\class Chat
	\brief Okno rozmowy
**/
class Chat : public QWidget
{
	Q_OBJECT

	private:
		friend class ChatManager;
		/**
		\struct RegisteredButton struktura przechowujca informacje o zarejestrowanym przycisku
		**/
		struct RegisteredButton
		{
			QString name; /*!< nazwa przycisku */
			QObject* receiver; /*!< obiekt z ktrego wykorzystana
					bdzie funkcja do obsugi przycisku */
			QString slot; /*!< funkcja podpita pod obsug
						tego przycisku */
		};
		static QValueList<RegisteredButton> RegisteredButtons;/*!<
				zarejestrowane przyciski w oknie rozmowy */
		QMap<QString,QPushButton*> Buttons; /*!< przyciski z okna */
		
		QValueList<ChatMessage *> ChatMessages; /*!< wiadomoci wpisane 
							w oknie rozmowy */
	
		UinsList Uins; /*!< lista uytkownikw w danym oknie */
		int index;	/*!< nr okna (z chat menadera) */
		QString title_buffer; /*!< tytu okna */
		QTimer* title_timer;  /*!< zmienna przechowujca czas od ostatniego 
					odwieenia tytuu okna */
		QColor actcolor; /*!< zmienna przechowujca aktualny kolor */
		
		QMimeSourceFactory *bodyformat; /*!< zmienna ustawiajcy format */
		EmoticonSelector* emoticon_selector; /*!< okienko do 
							wyboru emotikonek */
		ColorSelector* color_selector; /*!< okienko do wyboru koloru */
		QPushButton* boldbtn; /*!< przycisk pogrubiajcy czcionk */
		QPushButton* italicbtn; /*!< przycisk ustawiajcy 
					czcionk na kursyw */			
		QPushButton* underlinebtn; /*!< przycisk podkrelajcy tekst */
		QPushButton* colorbtn; /*!< przycisk umoliwiajcy 
					wybr koloru czcionki*/
		QPushButton* iconsel;
		QPushButton* autosend; /*!< przycisk automatycznego wysyania */
		QPushButton* lockscroll; /*!< przisk blokowania 
						przesuwania rozmowy */
		QAccel* acc; /*!< zmienna potrzebna do
				 rejestracji skrtw klawiszowych*/
		QPushButton* sendbtn; /*!< przycisk do wysyania wiadomoci */
		UserBox* userbox; /*!< lista kontaktw przydatna gdy jestemy w
						 konferencji */
		QString myLastMessage;/*!< zmienna przechowujca 
					nasz ostatni wiadomo */
		unsigned int myLastFormatsLength; /* ?? */
		void* myLastFormats; /* ?? */
		int seq; /* ?? */
		KaduSplitter *vertSplit, *horizSplit; /*!< obiekty oddzielajce
							 kontrolki od siebie */
		int ParagraphSeparator; /* ?? */
		QDateTime lastMsgTime; /*!< czas ostatniej wiadomoci */

		/**
			\fn void pruneWindow()
			Funkcja czyszczca okno rozmowy
		**/
		void pruneWindow(); 

	private slots:
		
		/**
			\fn void userWhois()
			Slot wywietlajcy informacje o zaznaczonym kontakcie
		**/
		void userWhois();
		
		/**
			\fn void emoticonSelectorClicked()
			Slot wywietlajcy okno z wyborem emotikonek
		**/
		void emoticonSelectorClicked();
		
		/**
			\fn void changeColor()
			Slot zmieniajcy kolor czcionki
		**/
		void changeColor();
		
		/**
			\fn void addMyMessageToHistory()
			Slot dodaje wysan wiadomo do historii rozmowy
		**/
		void addMyMessageToHistory();
		
		/**
			\fn void clearChatWindow()
			Slot czyszczcy okno rozmowy
		**/
		void clearChatWindow();
		
		/**
			\fn void pageUp()
			Slot przewijajcy histori rozmowy w gr
		**/
		void pageUp();
		
		/**
			\fn void pageDown()
			Slot przewijajcy histori rozmowy w d
		**/
		void pageDown();
		
		/**
			\fn void insertImage()
			Slot wklejajcy obrazek do okna rozmowy (wybr obrazka)
		**/
		void insertImage();
		
		/**
			\fn void imageReceivedAndSaved(UinType sender,uint32_t size,uint32_t crc32,const QString& path)
			Slot obsugujcy odebranie i zapis obrazka
			\param sender osoba, ktra wysaa obrazek
			\param size rozmiar obrazka
			\param crc32 suma kontrolna obrazka
			\param path cieka do obrazka
		**/
		void imageReceivedAndSaved(UinType sender,uint32_t size,uint32_t crc32,const QString& path);

		/**
			\fn void connectAcknowledgeSlots();
			Slot podczajcy potwierdzenie dostarczenia wiadomoci
		**/
		void connectAcknowledgeSlots();
		
		/**
			\fn void disconnectAcknowledgeSlots();
			Slot odczajcy potwierdzenie dostarczenia wiadomoci
		**/
		void disconnectAcknowledgeSlots();

	protected:
		/**
			\fn virtual void closeEvent(QCloseEvent* e)
			Funkcja obsugujca zamknicie okna
		**/
		virtual void closeEvent(QCloseEvent* e);
		
		/**
			\fn QString convertCharacters(QString text, const QColor &bgcolor, EmoticonsStyle style)
			Funkcja zamienia specjalne tagi na emotikonki,
			html na czysty tekst itp.
			\param text tekst do konwersji
			\param bgcolor kolor ta
			\param style styl emotikonek
		**/
		QString convertCharacters(QString text, const QColor &bgcolor, EmoticonsStyle style);
		
		/**
			\fn virtual void windowActivationChange(bool oldActive)
			Funkcja sterujca mruganiem napisu okna
		**/
		virtual void windowActivationChange(bool oldActive);
		
		/**
			\fn virtual void keyPressEvent(QKeyEvent* e)
			Funkcja obsugujca nacinicie przycisku
			\param e wskanik do obiektu obsugujcego klawisze
		**/
		virtual void keyPressEvent(QKeyEvent* e);
		
		/**
			\fn virtual void keyReleaseEvent(QKeyEvent* e)
			Funkcja obsugujca zwolnienie przycisku
			\param e wskanik do obiektu obsugujcego klawisze
		**/
		virtual void mouseReleaseEvent(QMouseEvent *e);

	public:
		// FIXME - nie powinno byc publicznych zmiennych
		KaduTextBrowser* body; /*!< historia rozmowy */
		CustomInput* edit; /*!< okno do wpisywania wiadomoci */
		QHBox* buttontray;/* ?? */
		
		/**
			Konstruktor okna rozmowy
			\fn Chat(UinsList uins, QWidget* parent = 0, const char* name = 0)
			\param parent rodzic okna
			\param name nazwa obiektu
		**/
		Chat(UinsList uins, QWidget* parent = 0, const char* name = 0);
		
		/**
			\fn ~Chat()
			Destruktor okna rozmowy
		**/
		~Chat();
		
		/**
			\fn static void registerButton(const QString& name, QObject* receiver, const QString& slot)
			Rejestruje przycisk w oknie chat, ustawia funkcje do obsugi 
			sygnau "clicked"
			\param name nazwa przycisku aby moc si do niego odwoa
			\param receiver obiekt ktry zawiera funkcje do obsugi zdarzenia
			\param slot funkcja ktra bdzie obsugiwa zdarzenie "clicked"
		**/
		static void registerButton(const QString& name, QObject* receiver, const QString& slot);
		
		/**
			\fn static void unregisterButton(const QString& name)
			Wyrejestrowuje przycisk ze wszystkich okien
			\param name nazwa przycisku do wyrejestrowania
		**/
		static void unregisterButton(const QString& name);
		
		/**
			\fn QPushButton* button(const QString& name) const
			Zwraca wskanik do przycisku 
			\param name nazwa przycisku
			\return wskanik do przycisku, jeli przycisk 
			nie istnieje zwraca NULL
		**/
		QPushButton* button(const QString& name) const;
		
		/**
			\fn void formatMessages(QValueList<ChatMessage *> &msgs)
			Formatuje wszystkie wiadomoci na html'a,
			dodaje kolory ta i czcionek
			\param msgs lista wiadomoci do sformatowania
		**/
		void formatMessages(QValueList<ChatMessage *> &msgs);
		/**
			\fn void formatMessage(ChatMessage &msg,
				QColor myBgColor=QColor(), QColor usrBgColor=QColor(),
				QColor myFontColor=QColor(), QColor usrFontColor=QColor(),
				EmoticonsStyle style=(EmoticonsStyle)config_file.readNumEntry("Chat","EmoticonsStyle"))

			Formatuje wiadomo na html'a
			\param msg wiadomo ktra bdzie sformatowana
			\param myBgColor kolor ktry bdzie ustawiony jako to
			w naszych wiadomociach
			\param usrBgColor kolor, ktry bdzie ustawiony jako to
			wiadomoci naszego rozmwcy
			\param myFontColor kolor naszej czcionki
			\param usrFontColor kolor czcionki naszego rozmwcy
			\param style styl ikonek 
		**/
		void formatMessage(ChatMessage &msg,
				QColor myBgColor=QColor(), QColor usrBgColor=QColor(),
				QColor myFontColor=QColor(), QColor usrFontColor=QColor(),
				EmoticonsStyle style=(EmoticonsStyle)config_file.readNumEntry("Chat","EmoticonsStyle"));
		
		/**
			\fn void checkPresence(UinsList, const QString&, time_t, QValueList<ChatMessage *> &messages)
			Dodaje wiadomo do okna
			\param senders lista uytkownikw
			\param msg wiadomoc
			\param time czas 
			\param messages lista wiadomoci do wpisania do okna
		**/
		void checkPresence(UinsList senders, const QString &msg, time_t time, QValueList<ChatMessage *> &messages);
		
		/**
			\fn void writeMessagesFromHistory(UinsList senders, time_t time)
			Wpisuje do okna histori rozmowy
			\param senders lista uytkownikw identyfikujcych okno
			\param time ..
		**/
		void writeMessagesFromHistory(UinsList senders, time_t time);
		
		/**
			\fn const UinsList& uins() const
			Zwraca list numerw rozmowcw.
		**/
		const UinsList& uins() const;
		
		/**
			\fn QValueList<ChatMessage*>& chatMessages()
			Daje dostp do wiadomoci aktualnie przechowywanych
			w oknie chat. Metody tej mona uy do zmiany treci
			lub waciwoci ktrej z wiadomoci w odpowiedzi
			na jakie zdarzenie.
		**/
		QValueList<ChatMessage*>& chatMessages();
		
		/**
			\fn void repaintMessages()
			Odwiea zawarto okna uwzgldniajc ewentualne
			zmiany dokonane w ktrej wiadomoci z listy
			uzyskanej za pomoc metody chatMessages(),
			dodanie nowych wiadomoci lub usunicie istniejcych.
		**/
		void repaintMessages();
		
		/**
			\fn const QString& title() const
			Zwraca aktualny tytu okna
		**/
		const QString& title() const;

	public slots:
		
		/**
			\fn void changeAppearance()
			Slot zmieniajcy kolory i czcionki w oknie
		**/
		void changeAppearance();
		
		/**
			\fn void setTitle()
			Slot ustawiajcy tytu okna zgodnie z konfiguracj
		**/
		void setTitle();
		
		/**
			\fn void addEmoticon(QString)
			Slot dodaje ikonke do pola wpisywania wiadomoci
			\param emot ikonka np. <lol>
		**/
		void addEmoticon(QString);
		
		/**
			\fn void scrollMessages(const QValueList<ChatMessage *> &)
			Slot dodaj wiadomoci do okna
			\param messages lista wiadomoci
		**/
		void scrollMessages(const QValueList<ChatMessage *> &);
		
		/**
			\fn void alertNewMessage()
			Slot informujcy o nowej wiadomoci 
			mruganie jeli okno jest nieaktywne
		**/
		void alertNewMessage();
		
		/**
			\fn void HistoryBox()
			Slot wywoywany po naciniciu przycisku historii
		**/
		void HistoryBox();
		
		/**
			\fn void sendMessage()
			Slot wywoywany po naciniciu przycisku 
			do wysyania wiadomoci
		**/
		void sendMessage();
		
		/**
			\fn void cancelMessage()
			Slot wywoywany po nacinieciu przycisku anulowania
			wysyaniu wiadomoci
		**/
		void cancelMessage();
		
		/**
			\fn void writeMyMessage()
			Slot wpisujcy wiadomo do okna
			\see sendMessage
		**/
		void writeMyMessage();
		
		/**
			\fn void changeTitle()
			Slot zmieniajcy tytu 
			\see setTitle
		**/
		void changeTitle();
		
		/**
			\fn void toggledBold(bool on)
			Slot wywoywany przy naciniciu przycisku pogrubienia
			\param on ustawia pogrubienie
		**/
		void toggledBold(bool on);
		
		/**
			\fn void toggledItalic(bool on)
			Slot wywoywany przy naciniciu przycisku kursywy
			\param on ustawia kursyw
		**/
		void toggledItalic(bool on);
		
		/**
			\fn void toggledUnderline(bool on)
			Slot wywoywany przy naciniciu przycisku podkrelenia
			\param on ustawia podkrelenie
		**/
		void toggledUnderline(bool on);
		
		/**
			\fn void curPosChanged(int para, int pos)
			Slot wywoywany kiedy pozycja kursosa zmienia si
			\param  para nie obslugiwane
			\param pos nie obslugiwane
		**/
		void curPosChanged(int para, int pos);
		
		/**
			\fn void specialKeyPressed(int key)
			Slot zostaje wywoany gdy nacinito specjalny klawisz (skrt)
			odpowiadajcy np KEY_BOLD
			\param key warto nacinitego specjalnego klawisza
		**/
		void specialKeyPressed(int key);
		
		/**
			\fn void colorChanged(const QColor& color)
			Slot zostaje wywowany gdy uytkownik zmieni kolor czcionki
			ktr bdzie pisa
			\param color warto zmienionego koloru
		**/
		void colorChanged(const QColor& color);
		
		/**
			\fn void colorSelectorAboutToClose()
			Slot zostaje wywoany przy zamykaniu okna wyboru ikonek
		**/
		void colorSelectorAboutToClose();
		
		/**
			\fn void messageBlockedSlot(int seq, UinType uin)
			Slot zostaje wywoany gdy nasza wiadomo zostaa zablokowana
			przez serwer
			\param seq 
			\param uin
		**/
		void messageBlockedSlot(int seq, UinType uin);
		
		/**
			\fn void messageBoxFullSlot(int seq, UinType uin)
			Slot zostaje wywoany gdy skrzynka na wiadomoci
			zostaa zapeniona
			\param seq
			\param uin
		**/
		void messageBoxFullSlot(int seq, UinType uin);
		
		/**
			\fn void messageNotDeliveredSlot(int seq, UinType uin)
			Slot zostaje wywoany gdy wiadomo nie zostaa dostarczona
			\param seq
			\param uin
		**/
		void messageNotDeliveredSlot(int seq, UinType uin);
		
		/**
			\fn void messageAcceptedSlot(int seq, UinType uin)
			Slot zostaje wywoany gdy wiadomo zostaa poprawnie wysana
			\param seq
			\param uin
		**/
		void messageAcceptedSlot(int seq, UinType uin);
		
	signals:
		/**
			\fn void messageSendRequested(Chat* chat)
			Sygnal jest emitowany gdy uzytkownik wyda polecenie
			wyslania wiadomosci, np klikajac na guzik "wyslij".
			\param chat wskanik do okna ktre emituje sygna
		**/
		void messageSendRequested(Chat* chat);
		
		/**
			\fn void messageFiltering(const UinsList& uins, QCString& msg, bool& stop)
			Sygnal daje mozliwosc operowania na wiadomoci
			ktora ma byc wyslana do serwera juz w jej docelowej
			formie po konwersji z unicode i innymi zabiegami.
			Tresc wiadomosci mozna zmienic podmieniajac wskaznik
			msg na nowy bufor i zwalniajac stary (za pomoca free).
			Mozna tez przerwac dalsza jej obrobke ustawiajac
			wskaznik stop na true.
			\param uins lista uytkownikw
			\param msg wiadomo 
			\param stop zakoczenie dalszej obrbki sygnau
		**/
		void messageFiltering(const UinsList& uins, QCString& msg, bool& stop);
		
		/**
			\fn void messageSent(Chat* chat)
			Sygnal jest emitowany gdy zakonczy sie proces
			wysylania wiadomosci i zwiazanych z tym czynnosci.
			Oczywiscie nie wiemy czy wiadomosc dotarla.
			\param chat wskanik do okna rozmowy,
			 ktre emitowao sygna
		**/		
		void messageSent(Chat* chat);
};

/**
	\class ColoSelectorButton
	\brief Klasa do wyboru koloru za pomoc przycisku
**/
class ColorSelectorButton : public QPushButton
{
	Q_OBJECT

	private:
		QColor color;/*!< kolor czcionki */

	private slots:
		/**
			\fn void buttonClicked()
			Slot obsugujcy nacinicie przycisku
		**/
		void buttonClicked();

	public:
		/**
			\fn ColorSelectorButton(QWidget* parent, const QColor& qcolor, int width=1, const char *name=0)
			Konstruktor tworzcy okno do wyboru koloru
		**/
		ColorSelectorButton(QWidget* parent, const QColor& qcolor, int width=1, const char *name=0);

	signals:
		/**
			\fn void clicked(const QColor& color)
			Sygna zostaje wysany po wybraniu koloru
		**/
		void clicked(const QColor& color);
};

/**
	\clas ColorSelector
	\brief Klasa do wyboru kolorw
**/
class ColorSelector : public QWidget
{
	Q_OBJECT

	private slots:
		/**
			\fn void iconClicked(const QColor& color);
			Slot obsugujcy wybranie koloru
		**/
		void iconClicked(const QColor& color);

	protected:
		/**
			\fn void closeEvent(QCloseEvent*)
			Funkcja obsugujca zamknicie okna wyboru kolorw
		**/
		void closeEvent(QCloseEvent*);

	public:
		/**
			\fn ColorSelector(const QColor &defColor, QWidget* parent = 0, const char* name = 0)
			Konstruktor obiektu do wyboru kolorw
		**/
		ColorSelector(const QColor &defColor, QWidget* parent = 0, const char* name = 0);

	public slots:
		/**
			\fn void alignTo(QWidget* w)
		  	Slot wyrwnuje pozycj do widgeta
			\param w wkanik do obiektu ktorego bdziemy wyrwnywac
		**/
		void alignTo(QWidget* w);

	signals:
		/**
			\fn void aboutToClose()
			Sygna jest emitowany gdy zamykany jest wybr kolorw
		**/
		void aboutToClose();
		
		/**
			\fn void colorSelect(const QColor&)
			Sygna jest emitowany gdy wybrano kolor
			
		**/
		void colorSelect(const QColor&);
};
/**
	\class ChatSlots
	\brief Sloty do obsugi okna konfiguracji
**/
class ChatSlots :public QObject
{
	Q_OBJECT

	private:
		/**
			\fn void updatePreview()
			Odwieenie wygldu
		**/
		void updatePreview();
	public:
		/**
			Konstruktor obiektu
			\param parent rodzic obiektu
			\param name nazwa obiektu
		**/
		ChatSlots(QObject* parent=0, const char* name=0);

	public slots:
		/**
			\fn void onCreateConfigDialog()
			Slot jest wywoywany gdy otwierane jest okno konfiguracji
		**/
		void onCreateConfigDialog();
		
		/**
			\fn void onDestroyConfigDialog()
			Slot jest wywoywany gdy zamykane jest okno konfiguracji
		**/
		void onDestroyConfigDialog();
		
		/**
			\fn void chooseEmoticonsStyle(int index)
			Slot jest wywoywany gdy wybierany jest styl ikonek
			\param index nr stylu emotikonek
		**/
		void chooseEmoticonsStyle(int index);
		
		/**
			\fn void onPruneChat(bool toggled)
			Slot jest wywoywany gdy wybierane jest czy otwierane okno
			ma by pusty czy nie
			\param toggled 
		**/
		void onPruneChat(bool toggled);
		
		/**
			\fn void onFoldLink(bool toggled)
			\param toggled
		**/
		void onFoldLink(bool toggled);
		
		/**
			\fn void onBlockClose(bool toggled)
			Slot jest wywoywany gdy wybierany jest maksymalny czas
			blokowania zamknicia okna po otrzymaniu wiadomoci
			\param toggled
		**/
		void onBlockClose(bool toggled);
		
		/**
			\fn void chooseColor(const char* name, const QColor& color)
			Slot jest wywoywany gdy zmieniono kolor,
			aby uaktualni podgld 
			\param name nazwa koloru
			\param color kolor
		**/
		void chooseColor(const char* name, const QColor& color);
		
		/**
			\fn void chooseFont(const char* name, const QFont& font)
			Slot jest wywoywany gdy zmieniono czcionke,
			aby uaktualni podgld 
			\param name nazwa czcionki
			\param font czcionka
		**/
		void chooseFont(const char* name, const QFont& font);
		
		/**
			\fn void findAndSetWebBrowser(int selectedBrowser)
			Slot jest wywoywany gdy wybrano z listy przegldark
			\param selectedBrowser nr przegldarki
		**/
		void findAndSetWebBrowser(int selectedBrowser);
		
		/**
			\fn void findAndSetBrowserOption(int selectedOption)
			Slot jest wywoywany gdy aktywowane pole do wpisania opcji
			\param selectedOption wybrana opcja
		**/
		void findAndSetBrowserOption(int selectedOption);
		
		/**
			\fn static void initBrowserOptions(QComboBox *browserCombo, QComboBox *browserOptionsCombo, QLineEdit *browserPath)
			Slot ustawia przegldarki wraz z opcjami
			\param browserCombo
			\param browserOptionsCombo
			\param browserPath
		**/
		static void initBrowserOptions(QComboBox *browserCombo, QComboBox *browserOptionsCombo, QLineEdit *browserPath);
		
		/**
			\fn static void findBrowser(int selectedBrowser, QComboBox *browserCombo, QComboBox *browserOptionsCombo, QLineEdit *browserPath)
			Slot jest wywoywany gdy wybrano z listy przegldark
			\param selectedBrowser
			\param browserCombo
			\param browserOptionsCombo
			\param browserPath
		**/
		static void findBrowser(int selectedBrowser, QComboBox *browserCombo, QComboBox *browserOptionsCombo, QLineEdit *browserPath);
		
		/**
			\fn static void setBrowserOption(int selectedOption, QLineEdit *browserPathEdit, int chosenBrowser)
			Slot ustawia przegldarki wraz z opcjami
			\param selectedOption
			\param browsePathEdit
			\param chosenBrowser
		**/
		static void setBrowserOption(int selectedOption, QLineEdit *browserPathEdit, int chosenBrowser);
};

#endif
