#ifndef KADU_MISC_H
#define KADU_MISC_H

#include <qcstring.h>
#include <qdatetime.h>
#include <qdialog.h>
#include <qfiledialog.h>
#include <qfont.h>
#include <qfontinfo.h>
#include <qiconset.h>
#include <qimage.h>
#include <qlabel.h>
#include <qmap.h>
#include <qpair.h>
#include <qpixmap.h>
#include <qstringlist.h>
#include <qsocket.h>
#include <qstring.h>
#include <qtextbrowser.h>
#include <qtooltip.h>
#include <qvaluelist.h>
#include <qvariant.h>

#include <inttypes.h>

#include "config_file.h"
#include "gadu.h"

class QComboBox;
class QLineEdit;
class QMenuData;

/**
	\fn QString ggPath(const QString &subpath="")
	Zmienia ciek relatywn do katalogu z ustawieniami gg
	na ciek bezwzgledn uwzgldniajc zmienne rodowiskowe
	\c $HOME i \c $CONFIG_DIR.
	Funkcja ta jest przydatna, gdy chcemy si odwoa do jakiego
	pliku/katalogu znajdujcego si w katalogu ustawie aktualnej
	sesji Kadu.
	\param subpath cieka relatywna do katalogu z ustawieniami gg
	\return ciek bezwzgldn do pliku.
	
	<b>Przykad:</b>
	\code
	MessageBox::msg(ggPath("kadu.conf"));
	\endcode
	Wywietlone zostanie okienko z pen ciek do pliku konfiguracyjnego Kadu.
**/
QString ggPath(const QString &subpath="");

/**
	\fn QString dataPath(const QString &f = "", const char *argv0 = 0)
	Zwraca bezwzgldn ciek do danego pliku.
	Jeeli drugi parametr jest rny od 0, to funkcja prbuje najpierw ustali
	ciek na podstawie \arg argv0, ktre ma by rwne argv[0], oraz zmiennej PATH.
	\param f nazwa pliku (cieka relatywna do katalogu z danymi Kadu) do ktrego
			ma zosta zwrcona pena, bezwzgldna cieka.
	\param argv0 nazwa uruchomionego programu. Moe to by zarwno '<i>kadu</i>',
			jak i '<i>kadu-kopia2</i>'. Domylnie jest rwny 0.
	\return ciek bezwzgldn do pliku.
**/
QString dataPath(const QString &f = "", const char *argv0 = 0);

class UserListElement;

/**
	\fn QString cp2unicode(const unsigned char *buf)
	Podmienia w danym acuchu znaki z kodowania CP1250
	na kodowanie Unicode.
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QString cp2unicode(const unsigned char *);

/**
	\fn QCString unicode2cp(const QString &buf)
	Podmienia w danym acuchu znaki z kodowania Unicode
	na kodowanie CP1250.
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QCString unicode2cp(const QString &);

/**
	\fn QString latin2unicode(const unsigned char *buf)
	Podmienia w danym acuchu znaki z kodowania Latin2
	na kodowanie Unicode.
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QString latin2unicode(const unsigned char *);

/**
	\fn QCString unicode2latin(const QString &buf)
	Podmienia w danym acuchu znaki z kodowania Latin2
	na kodowanie Unicode.
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QCString unicode2latin(const QString &buf);

/**
	\fn QString unicode2std(const QString &buf)
	Podmienia w danym acuchu polskie znaki z kodowania Unicode
	na ich odpowiedniki w kodowaniu ISO8859-2, czyli zmienia
	\a  na \a n, \a  na \a o, \a  na s, itd.
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QString unicode2std(const QString &);

/**
	\fn QString unicodeUrl2latinUrl(const QString &buf)
	Zamienia kodowanie polskich znakw z utf-8 na kodowanie latin-2
	przy pomocy QUrl::encode.
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QString unicodeUrl2latinUrl(const QString &buf);

/**
	\fn QString unicode2latinUrl(const QString &buf)
	Zamienia polskie znaki na format latin2 "URL" (czyli do postaci %XY,
	akceptowanej przez adresy URL).
	\param buf acuch znakw do przekodowania
	\return przekodowany acuch.
**/
QString unicode2latinUrl(const QString &buf);

/**
	\fn QString printDateTime(const QDateTime &datetime)
	Formatuje dat i czas do postaci czytelnej czowiekowi,
	czyli <tt>HH:MM:SS (DD.MM.YYYY)</tt>.
	\param datetime konkretna data, ktr naley sformatowa
	\return sformatowan dat.
**/
QString printDateTime(const QDateTime &datetime);

/**
	\fn QString timestamp(time_t customtime = 0)
	Formatuje stempel czasowy w postaci 2 dat: aktualna oraz podana.
	Ma on posta: <tt>HH:MM:SS (DD.MM.YYYY) /S HH:MM:SS (DD.MM.YYYY)</tt>,
	gdzie pierwsza data oznacza aktualn dat pobieran podczas wykonywania tej funkcji,
	a druga data to data podana jako argument i sformatowatowana przez funkcj unicode2latinUrl.
	Znacznik \c /S jest zwykym acuchem '/S'.
	\param customtime czas podany w formie sekund od pocztku roku 1970 (tzw. '<I>unixtime</i>'),
			ktry jest sformatowany i zwrcony jako druga data.
	\return sformatowany stempel czasowy
**/
QString timestamp(time_t = 0);

/**
	\fn QDateTime currentDateTime()
	Funkcja suca atwemu pobieraniu aktualnej daty.
	\return aktualn dat.
**/
QDateTime currentDateTime();

/**
	\fn QString pwHash(const QString &tekst)
	Funkcja hashujca zadany acuch tak, aby nie by od atwy do odczytania
	dla czowieka, bez uprzedniego potraktowania go w odpowiedni sposb.
	\param tekst acuch ktry ma zosta zahashowany
	\return zahashowany acuch
**/
QString pwHash(const QString &tekst);

/**
	\fn QString translateLanguage(const QApplication *application, const QString &locale, const bool l2n)
	Tumaczy nazwy skrtowe jzykw na ich pene nazwy i vice versa, czyli przykadowo
	\c pl przetumaczy na \c Polski, a \c Polski przetumaczy na \c pl.
	Jeli podane wyraenie nie moe zosta odnalezione w tablicy tyumacze, wtedy zwracany jest
	odpowiednia nazwa dla jzyka angielskiego.
	\param application obiekt aplikacji Qt, ktry ma przetumaczy. Zwykle jest to \c qApp
	\param locale wyraenie, ktre ma zosta odpowiednio przetumaczone. Moe to by:
			\c pl, \c en, \c de, \c it, ale rwnie:
			\c Polski, \c Angielski, \c Niemiecki, \c Woski
	\param l2n okrela, czy parametr \a locale jest wersj 2-literow (\c true), czy pen nazw (\c false)
	\return sowo odpowiadajce podanemu argumentowi \a locale.
**/
QString translateLanguage(const QApplication *application, const QString &locale, const bool l2n);

/**
	\fn void openWebBrowser(const QString &link)
	Otwiera domyln przegldark i wybiera w niej podany adres.
	\param link adres URL, ktry ma by otwarty w przegldarce
**/
void openWebBrowser(const QString &link);

/**
	\fn QString formatGGMessage(const QString &msg, int formats_length, void *formats, UinType sender)
	Formatuje wiadomo protokou GG do postaci HTML (pogrbienie, kursywa, podkrelenie, kolor).
	\param msg tre wiadomoci
	\param formats_length dugo parametrw formatujcych (szczegy w dokumentacji libgadu)
	\param formats wskanik do struktur formatujcych (szczegy w dokumentacji libgadu)
	\param sender UIN kontaktu, ktry ma by uyty do sformatowania wiadomoci (przychodzcej)
	\return gotow wiadomo sformatowan w HTMLu.
**/
QString formatGGMessage(const QString &msg, unsigned int formats_length, void *formats, UinType sender);

/**
	\fn QString unformatGGMessage(const QString &msg, int &formats_length, void *&formats)
	Formatuje wiadomo z postaci HTML do formatu akceptowanego przez protok GG.
	\param msg tre wiadomoci ze znacznikami HTML
	\param formats_length zmienna typu integer, do ktrej zapisana zostanie dugo
			parametrw formatujcych (szczegy w dokumentacji libgadu)
	\param formats wskanik, pod ktrym zapisane zostan struktury formatujce
			wiadomo (szczegy w dokumentacji libgadu)
	\return tre wiadomoci z usunitymi znacznikami HTML (s one teraz zakodowane
			w \a formats i \a formats_length ).
**/
QString unformatGGMessage(const QString &msg, unsigned int &formats_length, void *&formats);

/**
	\fn QString parse(const QString &s, const UserListElement &ule, bool escape = true)
	Podmienia odpowiednie znaczniki w danym acuchu wedug danych podanego kontaktu.
	Dozwolone s nastpujce znaczniki podstawowe:
	\arg \c %%s status
	\arg \c %%t status w orginalnej (nie przetumaczonej) formie.
	\arg \c %%d opis (lub pusty acuch jeli opisu nie ma)
	\arg \c %%i adres IP (jeli znany, inaczej pusty acuch)
	\arg \c %%v adres DNS (jeli znany, inaczej pusty acuch)
	\arg \c %%o jeli kontakt ma uytkownika Kadu na licie, to ten znacznik podmieniany jest na pojedyncz spacj
	\arg \c %%p port (jeli znany, inaczej pusty acuch)
	\arg \c %%u numer UIN
	\arg \c %%n pseudonim
	\arg \c %%a nazwa wywietlana
	\arg \c %%f imi
	\arg \c %%r nazwisko
	\arg \c %%m numer telefonu komrkowego
	\arg \c %%g grupa (lub kilka grup oddzielonych przecinkiem)
	\arg \c %%e adres e-mail
	\arg \c %%x maksymalny rozmiar obrazka w KB
	\arg \c %%% pojedynczy znak '%'

	Ponadto mona stosowa nawiasy \c [] i \c {}, oraz apostrofy: \c `'.

	Nawiasy kwadratowe pozwalaj na uywanie swego rodzaju instrukcji warunkowej,
	ktra sprawdza czy wszystkie (bez wyjtku) znaczniki zawarte pomidzy tymi
	nawiasami zwrciy warto niepust. Jeli tak, to w miejsce nawiasw wstawiany
	jest po prostu acuch zawarty midzy nimi (z podmienionymi ju znacznikami),
	jeli natomiast ktry ze znacznikw zwrci warto pust (np. %o, co oznaczao
	by e kontakt nie ma uytkownika Kadu na licie), to w miejsce nawiasw wstawiana
	jest warto pusta.

	Nawiasy klamrowe natomiast sprawdzaj, czy nazwa pliku - podana jako pierwszy wyraz zawarty
	w tych nawiasach - istnieje. Jeli tak, to w miejsce tych nawiasw wstawiany jest cay acuch
	midzy nimi zawarty, za wyjtkiem tej pierwszej nazwy pliku (i odstpu po niej). Jeli natomiast
	plik nie istnieje, to wstawiany jest pusty acuch.

	Wreszcie apostrofy ` i '. Dziki nim mona wykona polecenie w powoce systemu, a to co zwrci
	ta komenda, zostanie wstawione w miejsce caego wyraenia midzy apostrofami (cznie z nimi samymi).

	\param s acuch ze znacznikami do podmiany
	\param ule obiekt reprezentujcy kontakt, ktrego dane bd podstawiane pod znaczniki
	\param escape warto logiczna decydujca o tym, czy znaczniki HTML maj zosta zinterpretowane (\c false)
			czy te potraktowa je jako zwyky tekst (\c true).
	\return acuch z podstawionymi danymi zamiast znacznikw
**/
QString parse(const QString &s, const UserListElement &ule, bool escape = true);

/**
	\fn void stringHeapSort(QStringList &c)
	\internal
	Funkcja sortujca odpowiednio list acuchw.
	\todo Jakie to dokadnie sortowanie?
**/
void stringHeapSort(QStringList &c);

/**
	\fn QStringList toStringList(const QString &e1, const QString &e2=QString(), const QString &e3=QString(), const QString &e4=QString(), const QString &e5=QString())
	czy dowoln liczb (nie wiksz ni 5) acuchw QString w jedn list QStringList.
	\param e1 pierwszy acuch, wymagany
	\param e2 drugi acuch opcjonalny
	\param e3 trzeci acuch opcjonalny
	\param e4 czwarty acuch opcjonalny
	\param e5 pity acuch opcjonalny
	\return obiekt reprezentujcy list podanych acuchw
**/
QStringList toStringList(const QString &e1, const QString &e2=QString(), const QString &e3=QString(), const QString &e4=QString(), const QString &e5=QString());

/**
	\fn void saveGeometry(const QWidget *w, const QString &section, const QString &name)
	Zapisuje wymiary i pooenie wskazanego okna do danej sekcji konfiguracyjnej,
	do klucza o podanej nazwie.
	\param w wskanik do obiektu, ktrego wymiary i pooenie maj zosta zapisane
	\param section sekcja pliku konfiguracyjnego, w ktrej ma si znale wpis
	\param name nazwa klucza, w jakim maj by zapisane te dane
**/
void saveGeometry(const QWidget *w, const QString &section, const QString &name);

/**
	\fn void loadGeometry(QWidget *w, const QString &section, const QString &name, int defaultX, int defaultY, int defaultWidth, int defaultHeight)
	aduje wymiary i pooenie z podanej sekcji i klucza i aplikuje je do podanego okna.
	\param w wskanik do obiektu, do ktrego odczytane wymiary i pooenie maj zosta zastosowane
	\param section sekcja pliku konfiguracyjnego, z ktrego dane bd czytane
	\param name nazwa klucza, z ktrego dane bd czytane
	\param defaultX domylna warto pozycji lewego grnego rogu okna w poziomie (gdy brakuje wpisu w pliku konfiguracyjnym)
	\param defaultY domylna warto pozycji lewego grnego rogu okna w pionie (gdy brakuje wpisu w pliku konfiguracyjnym)
	\param defaultWidth domylna warto szerokoci okna (gdy brakuje wpisu w pliku konfiguracyjnym)
	\param defaultHeight domylna warto wysokoci okna (gdy brakuje wpisu w pliku konfiguracyjnym)
**/
void loadGeometry(QWidget *w, const QString &section, const QString &name, int defaultX, int defaultY, int defaultWidth, int defaultHeight);

/**
	\fn QString toPlainText(const QString &text)
	Usuwa znaki nowego wiersza, oraz tagi HTMLa (wszystko co da si dopasowa jako wyraenie regularne: \c <.*> ).
	\param text acuch do przetworzenia
	\return acuch z usunitymi odpowiednimi znakami.
**/
QString toPlainText(const QString &text);

/**
	\var extern QFont *defaultFont
	Domylna czcionka Kadu.
**/
extern QFont *defaultFont;

/**
	\var extern QFontInfo *defaultFontInfo
	Wskanik do obiekt zawierajcego informacje dotyczce domylnej czcionki Kadu
**/
extern QFontInfo *defaultFontInfo;

/**
	\var extern QTextCodec *codec_cp1250
	Wskanik do kodeka CP1250.
**/
extern QTextCodec *codec_cp1250;

/**
	\var extern QTextCodec *codec_latin2
	Wskanik do kodeka Latin2.
**/
extern QTextCodec *codec_latin2;

/**
	Klasa reprezentujca okno wybierania opisw do statusw.
	\class ChooseDescription
	\brief Okno wyboru opisu
**/
class ChooseDescription : public QDialog
{
	Q_OBJECT
	public:
		/**
			\fn ChooseDescription(int nr, QWidget * parent=0, const char * name=0)
			Konstruktor tworzcy okno, wczytujcy ostatnio ustawiane opisy,
			oraz ustawiajcy odpowiedni dla statusu ikon w tym oknie.
			\param nr numer statusu, gdzie:
				\arg \c 1 oznacza status <tt>Dostpny z opisem</tt>
				\arg \c 3 oznacza status <tt>Zajty z opisem</tt>
				\arg \c 5 oznacza status <tt>Ukryty z opisem</tt>
				\arg \c 7 oznacza status <tt>Niedostpny z opisem</tt>
				\arg <tt>Kady inny</tt> oznacza status <tt>Niedostpny z opisem</tt>
			\param parent rodzic kontrolki (domylnie: 0)
			\param name nazwa kontrolki (domylnie: 0)
		**/
		ChooseDescription ( int nr, QWidget * parent=0, const char * name=0);

		/**
			\fn virtual ~ChooseDescription()
			Domylny destruktor. Ustawia opis (jeli kliknito OK).
		**/
		virtual ~ChooseDescription();

		/**
			\fn void getDescription(QString &dest)
			Funkcja suca do odczytu treci z pola edycji opisu.
			\param dest obiekt klasy QString, pod ktrym zapisana zostanie tre
			\warning funkcja nie zwraca acucha, a zapisuje go do podanego parametru
		**/
		void getDescription(QString &);

	private:
		QComboBox *desc;
		QLabel *l_yetlen;

	private slots:
		void okbtnPressed();
		void cancelbtnPressed();
		void updateYetLen(const QString&);
};

/**
	Klasa bardzo pomocna, gdy chcemy, aby Kadu udawao przez chwil przegldark WWW.
	Pozwala na stosunkowo atwy odczyt danych ze stron, ale take bardziej zaawansowane
	operacje, jak logowanie na stronach metod POST, czy te poprzez ciasteczka.
	\class HttpClient
	\brief Klient stron WWW
**/
class HttpClient : public QObject
{
	Q_OBJECT

	private:
		QSocket Socket;
		QString Host;
		QString Referer;
		QString Path;
		QByteArray Data;
		QByteArray PostData;
		int StatusCode;
		bool HeaderParsed;

		unsigned int ContentLength;
		bool ContentLengthNotFound;

		QMap<QString, QString> Cookies;

	private slots:
		void onConnected();
		void onReadyRead();
		void onConnectionClosed();

	public slots:
		/**
			\fn void setHost(const QString &host)
			Ustawia adres serwera, z ktrego bdziemy czyta stron.
			\param host adres hosta w postaci acucha
		**/
		void setHost(const QString &host);

		/**
			\fn void get(const QString &path)
			Wysya zapytanie GET do ustawionego wczeniej serwera.
			\param path cieka do dokumentu, ktry chcemy pobra z serwera

			<b>Przykad:</b>
			\code
			http_client->get("/galeria/index.html");
			\endcode
		**/
		void get(const QString &path);

		/**
			\fn void post(const QString &path, const QByteArray &data)
			Wysya do serwera zapytanie typu POST, dodajc do niego
			odpowiednie dane.
			\param path cieka do dokumentu, ktry chcemy pobra z serwera
			\param data parametry, ktre s przekazane do serwera - zwykle zawieraj
					wartoci zmiennych z form na uprzednio pobranej stronie.
		**/
		void post(const QString &path, const QByteArray &data);

		/**
			\overload void post(const QString &path, const QString &data)
		**/
		void post(const QString &path, const QString &data);

	public:
		/**
			\fn HttpClient()
			Standardowy konstruktor.
		**/
		HttpClient();

		/**
			\fn int status() const
			\return kod aktualnego statu klienta WWW, zgodny z protokoem HTTP (302, 500, itd).
		**/
		int status() const;

		/**
			\fn const QByteArray &data() const
			Funkcja dajca dostp do danych reprezentujcych kod pobranej strony.
			\return tre pobranej strony.
		**/
		const QByteArray &data() const;

		/**
			\fn static QString encode(const QString &text)
			Zamienia znaki nieakceptowalne w adresie strony na ich odpowiedniki w formie %XY.
			\param text acuch reprezentujcy adres strony, zawierajcy spacje, itp
			\return sformatowany acuch.
		**/
		static QString encode(const QString &text);

		/**
			\fn const QString &cookie(const QString &name) const
			Funkcja dajca dostp do danych zawartych w tzw. ciasteczkach.
			\param name nazwa ciasteczka
			\return warto dla danego ciasteczka.
		**/
		const QString &cookie(const QString &name) const;

		/**
			\fn const QString &cookie(const QString &name) const
			Funkcja dajca dostp do danych zawartych w tzw. ciasteczkach.
			\return obiekt klasy QMap zawierajcy sparowane nazwy ciasteczek z ich wartociami.
		**/
		const QMap<QString, QString> &cookies() const;

		/**
			\fn void setCookie(const QString &name, const QString &value)
			Ustawia warto danego ciasteczka.
			\param name nazwa ciasteczka
			\param value warto dla danego ciasteczka
		**/
		void setCookie(const QString &name, const QString &value);

	signals:
		/**
			\fn void finished()
			Sygna emitowany, gdy zapytanie zostanie sfinalizowane.
		**/
		void finished();

		/**
			\fn void redirected(QString link)
			Sygna emitowany, gdy klient HTTP zostanie przekierowany przez serwer hosta
			na inny adres.
			\param link adres strony na jaki klient zosta przekierowany
		**/
		void redirected(QString link);

		/**
			\fn void error()
			Sygna emitowany, gdy wystpi bd podczas prby sfninaliowania zapytania.
		**/
		void error();
};

/**
	Klasa reprezentujca dokument HTML. Przechowuje
	list elementw. Element moe by tagiem HTML
	albo kawakiem tekstu.
	\class HtmlDocument
	\brief Dokument HTML
**/
class HtmlDocument
{
	private:
		/**
			Struktura reprezentujca pojedynczy element dokumentu.
			\struct Element
			\brief Element dokumentu
		**/
		struct Element
		{
			QString text; /*!< tre elementu */
			bool tag; /*!< oznacza, czy element jest takiem HTML, czy zwykym tekstem */
		};
		QValueList<Element> Elements;
		void addElement(Element e);

	public:
		/**
			\fn static void escapeText(QString& text)
			Zamienia znaki interpretowane przez przegldarki HTML (tj. \c < , \c > , itd.)
			na ich odpowiedniki zakodowane w HTML (tj. \c &lt; , \c &gt; , itd.).
			\param text obiekt klasy QString zawierajcy znaki do podmiany
			\warning funkcja nie zwraca acucha z podmienionymi znakami,
					a zapisuje go od od razu do podanego obiektu, zmieniajc jego zawarto.
		**/
		static void escapeText(QString& text);

		/**
			\fn static void unescapeText(QString& text)
			Zamienia znaki zakodowane w HTML (tj. \c &lt; , \c &gt; , itd.)
			na ich odpowiedniki interpretowane przez przegldarki HTML (tj. \c < , \c > , itd.).
			\param text obiekt klasy QString zawierajcy znaki do podmiany
			\warning funkcja nie zwraca acucha z podmienionymi znakami,
					a zapisuje go od od razu do podanego obiektu, zmieniajc jego zawarto.
		**/
		static void unescapeText(QString& text);

		/**
			\fn void addTag(const QString &text)
			Dodaje podany tekst do listy elementw jako
			tag HTML.
			\param text tekst do dodania
		**/
		void addTag(const QString &text);

		/**
			\fn void addText(const QString &text)
			Dodaje podany tekst do listy elementw jako
			zwyky tekst.
			\param text tekst do dodania
		**/
		void addText(const QString &text);

		/**
			\fn void insertTag(const int pos,const QString &text)
			Wstawia podany tekst do listy elementw jako
			tag HTML przed innym elementem.
			\param pos indeks elementu, przed ktrym wstawiamy
			\param text tekst do dodania
		**/
		void insertTag(const int pos,const QString &text);

		/**
			\fn void insertText(const int pos,const QString &text)
			Wstawia podany tekst do listy elementw jako
			zwykly tekst przed innym elementem.
			\param pos indeks elementu, przed ktrym wstawiamy
			\param text tekst do dodania
		**/
		void insertText(const int pos,const QString &text);

		/**
			\fn void parseHtml(const QString& html)
			Parsuje podany napis zgodny ze struktur HTML
			i otrzymane dane przypisuje do dokumentu.
			@param html napis do skonwertowania
		**/
		void parseHtml(const QString& html);

		/**
			\fn QString generateHtml() const
			Na podstawie zawartoci dokumentu generuje
			napis zgodny ze struktur HTML. Znaki specjalne
			wchodzce w skad elementw nie bdcych tagami
			HTML s escapowane.
			\return zawarto dokumentu w formacie HTML.
		**/
		QString generateHtml() const;

		/**
			\fn int countElements() const
			\return ilo elementw wchodzcych w skad
			dokumentu.
		**/
		int countElements() const;

		/**
			\fn bool isTagElement(int index) const
			Sprawdza czy element o podanym indeksie jest
			tagiem HTML czy zwykym tekstem.
			\return \c true jeli jest tagiem HTML, lub \c false gdy jest zwykym tekstem.
		**/
		bool isTagElement(int index) const;

		/**
			\fn const QString &elementText(int index) const
			\return tekst elementu o podanym indeksie.
		**/
		const QString &elementText(int index) const;

		/**
			\overload QString &elementText(int index)
		**/
		QString &elementText(int index);

		/**
			\fn void setElementValue(int index,const QString& text,bool tag=false)
			Ustawia tekst i typ elementu o podanym indeksie.
			\param index indeks pozycji elementu
			\param text tre elementu
			\param tag czy element ma by interpretowany jako tag HTML (\c true ),
				czy jako zwyky tekst (\c false ). Domylnie: \c false.
		**/
		void setElementValue(int index,const QString& text,bool tag=false);

		/**
			\fn void splitElement(int& index,int start,int length)
			Wydziela z elementu podany fragment tekstu.
			Element moe pozosta jeden albo
			ulec podzieleniu na dwa lub nawet trzy elementy.
			\param index indeks pozycji elementu ktry bdziemy dzieli
			\param start numer znaku, od ktrego zaczyna ma si wydzielony element
			\param length liczba znakw wchodzcych w wydzielony element, poczynajc od pozycji \c start
			\note Funkcja zmienia warto argumentu index, aby
			indeks wydzielonego elementu by aktualny.
		**/
		void splitElement(int& index,int start,int length);

		/**
			\fn void convertUrlsToHtml()
			Skanuje tekst dokumentu w poszukiwaniu adresw WWW,
			np. "www.kadu.net" i zmienia je w linki HTML.
		**/
		void convertUrlsToHtml();
};

/*
	nie uywane - patrz opis w pliku .cpp

HtmlDocument GGMessageToHtmlDocument(const QString &msg, int formats_length, void *formats);
void HtmlDocumentToGGMessage(HtmlDocument &htmldoc, QString &msg,
	int &formats_length, void *&formats);
*/

/**
	Klasa reprezentujca kontrolki obrazkw.
	\class ImageWidget
	\brief Obrazki
**/
class ImageWidget : public QWidget
{
	Q_OBJECT

	private:
		QImage Image;

	protected:
		/**
			\fn virtual void paintEvent(QPaintEvent *e)
			Funkcja rysujca t kontrolk.
			\param e wskanik do obiektu reprezentujcego zdarzenie rysowania.
		**/
		virtual void paintEvent(QPaintEvent *e);

	public:
		/**
			\fn ImageWidget(QWidget *parent)
			Konstruktor tworzcy kontrolk w innej - podanej kontrolce.
			\param parent kontrolka, w ktrej naley obsadzi
		**/
		ImageWidget(QWidget *parent);

		/**
			\overload ImageWidget(QWidget *parent,const QByteArray &image)
			Konstruktor tworzcy kontrolk w innej - podanej kontrolce,
			oraz rysujcy od razu dany obrazek.
			\param parent kontrolka, w ktrej naley obsadzi
			\param image dane zawierajce obrazek
		**/
		ImageWidget(QWidget *parent,const QByteArray &image);

		/**
			\fn void setImage(const QByteArray &image)
			Ustawia obrazek kontrolki na zadany.
			\param image dane zawierajce obrazek.
		**/
		void setImage(const QByteArray &image);

		/**
			\overload void setImage(const QPixmap &image)
		**/
		void setImage(const QPixmap &image);
};

/**
	Klasa reprezentujca okienko z obrazkiem i polem edycji.
	Jest ona uywana do wywietlania obrazka z bramki SMS Idei,
	aby uytkownik mg odczyta z niego tre i wpisa j do pola edycji.
	\brief Odczytywanie treci z obrazka WWW
	\class TokenDialog
**/
class TokenDialog : public QDialog
{
	Q_OBJECT

	public:
		/**
			\fn TokenDialog(QPixmap tokenImage, QDialog *parent = 0, const char *name = 0)
			Konstruktor tworzcy okienko wraz z gotowym obrazkiem.
			\param tokenImage obrazek do wywietlenia
			\param parent rodzic kontrolki (domylnie: 0)
			\param name nazwa kontrolki (domylnie: 0)
		**/
		TokenDialog(QPixmap tokenImage, QDialog *parent = 0, const char *name = 0);

		/**
			\fn void getValue(QString &tokenValue)
			Pobiera tre pola edycji spod obrazka.
			\param tokenValue obiekt klasy QString, pod ktrym zostanie zapisana pobrana tre.
		**/
		void getValue(QString &tokenValue);

	private:
		QLineEdit *tokenedit;
};

/**
	Klasa zarzdzajca stylami wygldu Kadu.
	\class Themes
	\brief Style Kadu
**/
class Themes : public QObject
{
	Q_OBJECT
	private:
		QStringList ThemesList;
		QStringList ThemesPaths;
		QStringList additional;
		QString ConfigName, Name, ActualTheme;
		QMap<QString, QString> entries;
		QStringList getSubDirs(const QString& path) const;
		QString fixFileName(const QString& path,const QString& fn) const;

	public:
		/**
			\fn Themes(const QString& name, const QString& configname, const char *name=0)
			Konstruktor tworzcy grup styli Kadu.
			\param name nazwa grupy styli (\c emoticons , \c sounds , \c icons )
			\param configname nazwa pliku konfiguracyjnego
			\param name nazwa obiektu (domylnie: 0)
		**/
		Themes(const QString& name, const QString& configname, const char *name=0);

		/**
			\fn QStringList defaultKaduPathsWithThemes() const
			\return domyln list cieek, w ktrych Kadu trzyma grupy styli.
		**/
		QStringList defaultKaduPathsWithThemes() const;

		/**
			\fn const QStringList &themes() const
			\return list styli Kadu.
		**/
		const QStringList &themes() const;

		/**
			\fn const QStringList &themes() const
			\return bierzcy styl.
		**/
		const QString &theme() const;

		/**
			\fn const QStringList &paths() const
			\return list cieek, w ktrych Kadu trzyma grupy styli.
		**/
		const QStringList &paths() const;

		/**
			\fn const QStringList &additionalPaths() const
			\return list dodatkowych cieek, w ktrych Kadu trzyma grupy styli.
		**/
		const QStringList &additionalPaths() const;

		/**
			\fn QString themePath(const QString& theme="") const
			Pobiera ciek do danego stylu.
			\param theme nazwa stylu (domylnie: "", co oznacza styl domylny)
			\return ciek do katalogu podanego stylu
		**/
		QString themePath(const QString& theme="") const;

		/**
			QString getThemeEntry(const QString& name) const
			Pobiera warto danego wpisu.
			\param name nazwa klucza, dla ktrego naley pobra wpis
			\return wpis z danego stylu, o podanym kluczu
			\todo opisa dokadniej co to za wpisy.
		**/
		QString getThemeEntry(const QString& name) const;

	public slots:
		/**
			\fn void setTheme(const QString& theme)
			Ustawia aktualny styl.
			\param theme styl, ktry ma by ustawiony
		**/
		void setTheme(const QString& theme);

		/**
			\fn void setPaths(const QStringList& paths)
			Ustawia cieki dla styli.
			\param paths lista cieek
		**/
		void setPaths(const QStringList& paths);

	signals:
		/**
			\fn void themeChanged(const QString& theme)
			Sygna emitowany, gdy zmieniony zosta styl.
			\param theme nazwa nowego stylu
		**/
		void themeChanged(const QString& theme);

		/**
			\fn void pathsChanged(const QStringList& list)
			Sygna emitowany, gdy cieki do styli zostay zmienione.
			\param list lista nowych cieek
		**/
		void pathsChanged(const QStringList& list);
};

/**
	Klasa zarzdzajca zestawami ikon, oraz samymi ikonami.
	Istnieje globalna zmienna - icons_manager_ptr - tak wic tworzenie
	nowych instancji tego obiektu nie jest potrzebne.
	\class IconsManager
	\brief Menader ikon
**/
class IconsManager :public Themes
{
	Q_OBJECT
	public:
		/**
			\fn IconsManager(const QString& name, const QString& configname)
			Konstruktor tworzcy grup styli ikon.
			\param name nazwa grupy
			\param configname nazwa pliku konfiguracyjnego
		**/
		IconsManager(const QString& name, const QString& configname);

		/**
			\fn QString iconPath(const QString &name) const
			Tumaczy nazw ikony z zestawu na ciek do niej i na odwrt.
			\param name nazwa ikony z zestawu lub scieka do pliku
					(jeli zawiera znak '/' to jest interpretowana jako
					scieka).
			\return pen ciek do ikony z aktualnego zestawu,
					lub bezposrednio nazw jeli jest to pena cieka.
		**/
		QString iconPath(const QString &name) const;

		/**
			\fn const QPixmap & loadIcon(const QString &name)
			aduje ikon z aktualnego zestawu lub z podanego pliku.
			\param name nazwa ikony z zestawu lub cieka do pliku
					(jeli zawiera znak '/' to jest interpretowana jako
					cieka).
			\return obiekt klasy QPixmap reprezentujcy dan ikon.
		**/
		const QPixmap & loadIcon(const QString &name);

		/**
			\fn static void initModule()
			Inicjalizuje modu menadera ikon.
		**/
		static void initModule();

		/**
			\fn void registerMenu(QMenuData *menu)
			Rejestruje nowe menu.
			\param menu wskanik do obiektu danych dla menu
			\todo opisa dokadniej co to za menu.
		**/
		void registerMenu(QMenuData *menu);

		/**
			\fn void registerMenu(QMenuData *menu)
			Wyrejestruje menu.
			\param menu wskanik do obiektu danych menu
			\todo opisa dokadniej co to za menu.
		**/
		void unregisterMenu(QMenuData *menu);

		/**
			\fn void registerMenuItem(QMenuData *menu, const QString &caption, const QString &iconName)
			Rejestruje nowy element w menu.
			\param menu wskanik do obiektu danych menu
			\param caption tytu nowego elementu
			\param iconName nazwa ikony dla nowego elementu
		**/
		void registerMenuItem(QMenuData *menu, const QString &caption, const QString &iconName);

		/**
			\fn void unregisterMenuItem(QMenuData *menu, const QString &caption)
			Wyrejestrowuje element z menu.
			\param menu wskanik do obiektu danych menu
			\param caption tytu elementu
		**/
		void unregisterMenuItem(QMenuData *menu, const QString &caption);

	public slots:
		/**
			\fn void clear()
			Czyci zestaw ikon.
		**/
		void clear();

		/**
			\fn void refreshMenus()
			Odwierza menu.
		**/
		void refreshMenus();

	private:
		QMap<QString, QPixmap> icons;
		
		QValueList<QPair<QMenuData *, QValueList<QPair<QString, QString> > > > menus;

	private slots:
		void selectedPaths(const QStringList& paths);
		void onCreateConfigDialog();
		void onApplyConfigDialog();
};

/**
	\var extern IconsManager *icons_manager_ptr
	Wskanik do obiektu generalnego zarzdcy ikonek Kadu.
**/
//TODO: po wydaniu 0.4 trzeba zmieni nazw na icons_manager i wywali define'a
extern IconsManager *icons_manager_ptr;
/**
	\def icons_manager
	Tymczasowy skrt bezporednio do obiektu generalnego zarzdcy ikonek Kadu,
	zamiast wskanika icons_manager_ptr.
**/
#define icons_manager (*icons_manager_ptr)


/**
	Klasa rozwiazujaca problem z powiadomieniem
	o utworzeniu nowej instancji danej klasy.
	Umieszczamy w klasie publiczna statyczna
	zmienna createNotifier klasy CreateNotifier,
	do ktorej mog si podcza pozostae czeci kodu.
	Przed wyjciem z konstruktora wywoujemy metod:
	\code createNotifier.notify(this); \endcode
	\class CreateNotifier
	\brief Powiadamianie o nowych instancjach klas
**/
class CreateNotifier : public QObject
{
	Q_OBJECT

	public:
		/**
			\fn void notify(QObject* new_object)
			Emituje sygna do wszystkich obiektw podczonych do tego obiektu
			informujcy o tym, e dany obiekt zosta utworzony. Funkcja ta powinna
			by wywoana przez nowotworzony obiekt, w jego konstruktorze.
			\param new_object wskanik do obiekty, ktry zosta utworzony (zwykle podaje si \c this )
		**/
		void notify(QObject* new_object);

	signals:
		/**
			\fn void objectCreated(QObject* new_object)
			Sygna emitowany, gdy jaki obiekt wywoa metod notify() (zwykle z parametrem \c this ).
			\param new_object wskanik do nowoutworzonego obiektu
		**/
		void objectCreated(QObject* new_object);
};

/**
	Klasa zarzdzajca obrazkami przesyanymi przez protok GG.
	\class GaduImagesManager
	\brief Zarzdca obrazkw protokou GG
**/
class GaduImagesManager
{
	private:
		struct ImageToSend
		{
			uint32_t size;
			uint32_t crc32;
			QString file_name;
			QDateTime lastSent;
			char *data;
		};
		QMap<QPair<uint32_t, uint32_t>, ImageToSend> ImagesToSend;//size,crc32 -> 

		struct SavedImage
		{
			uint32_t size;
			uint32_t crc32;
			QString file_name;
		};
		QValueList<SavedImage> SavedImages;

	public:
		/**
			\fn static void setBackgroundsForAnimatedImages(HtmlDocument &doc, const QColor &col)
			Ustawia kolor ta dla animowanych obrazkw w danym dokumencie.
			\param doc dokument, w ktrym kolory zostan ustawione
			\param col kolor ta
		**/
		static void setBackgroundsForAnimatedImages(HtmlDocument &doc, const QColor &col);

		/**
			\fn static QString loadingImageHtml(UinType uin, uint32_t size, uint32_t crc32)
			Generuje odpowiedni kod HTML, ktry naley wstawi do okna rozmowy, aby pokaza,
			e obrazek jest w trakcie adowania (przesyania), oraz ustawia
			odpowiednie parametry znacznika &lt;img&gt; tak, aby mona byo potem atwo
			zidentyfikowa, jaki obrazek mia by wstawiony w to miejsce.
			W praktyce daje nam znan klepsydr.
			\param uin UIN kontaktu wysyajcego obrazek
			\param size rozmiar obrazka
			\param crc32 suma kontrolna obrazka
			\return acuch stanowicy kod HTML
		**/
		static QString loadingImageHtml(UinType uin, uint32_t size, uint32_t crc32);

		/**
			\fn static QString imageHtml(const QString& file_name)
			Generuje kod HTML, ktry naley wstawi do okna rozmowy,
			aby pokaza w nim obrazek.
			\param file_name nazwa pliku obrazka
			\return acuch stanowicy kod HTML
		**/
		static QString imageHtml(const QString& file_name);

		/**
			\fn void addImageToSend(const QString& file_name, uint32_t& size, uint32_t& crc32)
			Dodaje obrazek do listy obrazkw, ktre maj by wysane z kolejn wiadomoci.
			\param file_name nazwa pliku obrazka
			\param size rozmiar obrazka
			\param crc32 suma kontrolna obrazka
		**/
		void addImageToSend(const QString& file_name, uint32_t& size, uint32_t& crc32);

		/**
			\fn void sendImage(UinType uin, uint32_t size, uint32_t crc32)
			Wysya jeden z obrazkw znajdujcych si na uprzednio przygotowanej licie.
			\param uin numer UIN do ktre ma by wysany obrazek
			\param size rozmiar obrazka, ktry chcemy wysa
			\param crc32 suma kontrolna obrazka, ktry chcemy wysa
		**/
		void sendImage(UinType uin, uint32_t size, uint32_t crc32);

		/**
			\fn QString getImageToSendFileName(uint32_t size, uint32_t crc32)
			Szuka zakolejkowanego obrazka.
			\return nazw pliku znalezionego obrazka. Zwraca cig pusty,
			jeli obrazek nie zosta w tej sesji zakolejkowany do wysania.
		**/
		QString getImageToSendFileName(uint32_t size, uint32_t crc32);

		/**
			\fn QString saveImage(UinType sender, uint32_t size, uint32_t crc32, const QString& filename, const char* data)
			Zapisuje obrazek w katalogu .gg/images.
			\return pen ciek do zapisanego obrazka.
		**/
		QString saveImage(UinType sender, uint32_t size, uint32_t crc32, const QString& filename, const char* data);

		/**
			\fn QString getSavedImageFileName(uint32_t size, uint32_t crc32)
			Szuka zapisanego obrazka.
			\return zwraca jego nazw pliku wraz ze ciek. Zwraca cig pusty,
			jeli obrazek nie zosta w tej sesji zapisany.
		**/
		QString getSavedImageFileName(uint32_t size, uint32_t crc32);

		/**
			\fn QString replaceLoadingImages(const QString& text, UinType sender, uint32_t size, uint32_t crc32)
			Podmienia obrazki klepsydr na waciwy obrazek, o ile zosta on ju w caoci pobrany.
			Funkcj t naley wywoa dla kadego niepowtarzalnego obrazka w treci dokumentu.
			\param text tre dokumentu ze znacznikami HTML oznaczajcymi klepsydry
			\param sender UIN kontaktu ktry przysya obrazki
			\param size rozmiar obrazka ktry chcemy wstawi
			\param crc32 suma kontrolna obrazka ktry chcemy wstawi
			\return tre dokumentu z podmienionymi znacznikami HTML
		**/
		QString replaceLoadingImages(const QString& text, UinType sender, uint32_t size, uint32_t crc32);
};

/**
	\var extern GaduImagesManager gadu_images_manager
	Obiekt zarzdzajcy obrazkami przesyanymi przez protok Gadu-Gadu.
**/
extern GaduImagesManager gadu_images_manager;

/**
	Klasa suaca do pogldu obrazkw podczas wyboru pliku obrazka.
	\class PixmapPreview
	\brief Podgld obrazkw
**/
class PixmapPreview : public QLabel, public QFilePreview
{
	public:
		/**
			\fn PixmapPreview()
			Domylny konstruktor.
		**/
		PixmapPreview();

		/**
			\fn void previewUrl(const QUrl& url)
			Ustawia podgld dla danego pliku.
			\param url adres (cieka lokalna lub plik w sieci) pliku obrazka
		**/
		void previewUrl(const QUrl& url);
};

/**
	Klasa reprezentujca okno wyboru obrazka do wstawiania.
	\class ImageDialog
	\brief Okno wyboru obrazka
**/
class ImageDialog : public QFileDialog
{
	public:
		/**
			\fn ImageDialog(QWidget* parent)
			Konstruktor tworzcy okno danym rodzicem.
			\param parent rodzic kontrolki
		**/
		ImageDialog(QWidget* parent);
};

/**
	Zmodyfikowany QTextBrowser specjalnie na potrzeby Kadu.
	Kliknicie na linku otwiera ustawion w konfiguracji przegldark.
	W menu kontekstowym jest dodatkowa opcja "Kopiuj lokacj odnonika".
	Dodatkowo poprawka bdu w Qt.
	\class KaduTextBrowser
	\brief Wywietlanie wiadomoci
**/
class KaduTextBrowser : public QTextBrowser, QToolTip
{
	Q_OBJECT

	private:
		QString anchor;
		int level;
		/**
			this value stores the current highlighted link
			for use with maybeTip(), or is null
		**/
		QString highlightedlink; 	

	private slots:
		void copyLinkLocation();
		void hyperlinkClicked(const QString& link) const;
		void linkHighlighted(const QString &);
		
	protected:
		/**
			\fn QPopupMenu *createPopupMenu(const QPoint &point)
			Wywietla menu kontekstowe.
			\param point wsprzdne punktu, w ktrym ma znale si lewy grny rg menu kontekstowego
			\return wskanik do obiektu menu
		**/
		QPopupMenu *createPopupMenu(const QPoint &point);

		/**
			\fn virtual void drawContents(QPainter * p, int clipx, int clipy, int clipw, int cliph)
			Rysuje zawarto kontrolki (wiadomoci, separatory, itp).
			\param p wskanik do obiektu rysujcego
			\param clipx wsprzdne poziome lewego grnego rogu prostokta ktry zostanie narysowany
			\param clipy wsprzdne pionowe lewego grnego rogu prostokta ktry zostanie narysowany
			\param clipw szeroko prostokta ktry zostanie narysowany
			\param cliph wysoko prostokta ktry zostanie narysowany
		**/
		virtual void drawContents(QPainter * p, int clipx, int clipy, int clipw, int cliph);

		/**
			\fn virtual void maybeTip(const QPoint& point)
			Funkcja wywietlajca podpowied.
			\param point wsprzdne okrelajce lewy grny rg podpowiedzi
		**/
		virtual void maybeTip(const QPoint&);

		/**
			\fn virtual void contentsMouseReleaseEvent(QMouseEvent * e)
			Emituje sygna puszczenia przycisku myszy.
			\param e wskanik do obiektu reprezentujcego zdarzenie myszy
		**/
		virtual void contentsMouseReleaseEvent(QMouseEvent * e);

	public:
		/**
			\fn KaduTextBrowser(QWidget *parent = 0, const char *name = 0)
			Konstruktor standardowy.
			\param parent rodzic kontrolki (domylnie: 0)
			\param name nazwa kontrolki (domylnie: 0)
		**/
		KaduTextBrowser(QWidget *parent = 0, const char *name = 0);

		/**
			\fn void setSource(const QString &name)
			Ustawia rdo danych.
			\param name nazwa rda danych
			\note Tak na prawd funkcja ta na obecn chwil nie robi nic.
		**/
		void setSource(const QString &name);

		/**
			\fn void clear()
			Czyci zawarto.
		**/
		void clear() 
		{ 
			setText("");
			//dla QT<3.2 clear() powodowao bd 
		}

		/**
			\fn void setFont(const QFont& f)
			Ustawia czcionk.
			\param f obiekt czcionki do ustawienia
		**/
		void setFont(const QFont& f) 	{ QTextBrowser::setFont(f); }

		/**
			\fn void setMargin(int width)
			Ustawia szeroko marginesu.
			\param width liczba pikseli
		**/
		void setMargin(int width);

	public slots:
		/**
			\fn virtual void copy()
			Kopiuje zaznaczon cz do schowka.
		**/
		virtual void copy();

	signals:
		/**
			\fn void mouseReleased(QMouseEvent *e, KaduTextBrowser *sender)
			Dowolny przycisk myszy zosta zwolniony.
			\param e wskanik do obiektu reprezentujcego zdarzenie myszy
			\param sender wskanik do obiektu, ktry wyemitowa ten sygna, czyli \c this
		**/
		void mouseReleased(QMouseEvent *e, KaduTextBrowser *sender);
};

/**
	\fn QValueList<int> toIntList(const QValueList<QVariant> &in)
	Konwertuje ca list wariantw do listy liczb cakowitych za jednym zamachem.
**/
QValueList<int> toIntList(const QValueList<QVariant> &in);

/**
	\fn QValueList<int> toIntList(const QValueList<QVariant> &in)
	Konwertuje ca list liczb cakowitych do listy wariantw za jednym zamachem.
**/
QValueList<QVariant> toVariantList(const QValueList<int> &in);

/**
	\fn template<class T, class X> QValueList<T> keys(const QMap<T, X> &m)
	Funkcja ta jest reimplementacj funkcji z Qt - QMap::keys(), w celu
	ominicia bdu krytycznego w Qt.
**/
template<class T, class X> QValueList<T> keys(const QMap<T, X> &m)
{
#if QT_VERSION < 0x030005
    QValueList<T> ret;
    for(QMap<T,X>::const_iterator it = m.begin(), map_end = m.end(); it != map_end; ++it)
        ret.append(it.key());
    return ret;
#else
    return m.keys();
#endif
}

/**
	\fn template<class T, class X> QValueList<X> values(const QMap<T, X> &m)
	Funkcja ta jest reimplementacj funkcji z Qt - QMap::values(), w celu
	ominicia bdu krytycznego w Qt.
**/
template<class T, class X> QValueList<X> values(const QMap<T, X> &m)
{
#if QT_VERSION < 0x030005
    QValueList<X> ret;
    for(QMap<T,X>::const_iterator it = m.begin(), map_end = m.end(); it != map_end; ++it)
        ret.append(it.data());
    return ret;
#else
    return m.values();
#endif
}

/**
	\fn QString narg(const QString &s, const QString &arg1, const QString &arg2, const QString &arg3=QString(), const QString &arg4=QString())
	Zastpstwo dla arga z QString, ktre podmienia kolejne %[1-4] w miejscu.

	W QStringu efektem:
	\code QString("%1 odstp %2").arg("pierwszy %1 tekst").arg("drugi tekst") \endcode
	jest <tt>"pierwszy drugi tekst tekst odstp %2"</tt>
	a chcielibymy eby byo
	<tt>"pierwszy %1 tekst odstp drugi tekst"</tt>
	co robi wanie ta funkcja.
**/
QString narg(const QString &s, const QString &arg1, const QString &arg2,
				const QString &arg3=QString(), const QString &arg4=QString());

/**
	\overload QString narg(const QString &s, const QString &arg1, const QString &arg2, const QString &arg3, const QString &arg4, const QString &arg5, const QString &arg6=QString(), const QString &arg7=QString(),const QString &arg8=QString(), const QString &arg9=QString())
**/
QString narg(const QString &s, const QString &arg1, const QString &arg2,
				const QString &arg3, const QString &arg4,
				const QString &arg5, const QString &arg6=QString(),
				const QString &arg7=QString(),const QString &arg8=QString(),
				const QString &arg9=QString());

/**
	\overload QString narg(const QString &s, const QString **tab, int count)
	Uoglniony narg() na wiksz liczb parametrw.
	\param count musi by mniejszy lub rwny 9
	\param tab tablica \c count wskanikw do QString
**/
QString narg(const QString &s, const QString **tab, int count);

/**
	\fn void printBacktrace(const QString &header="")
	Wypisuje na wyjciu standardowym tzw. <i>backtrace</i>.
	\param header opcjonalna nazwa nagwka
**/
void printBacktrace(const QString &header="");

/**
	\def VAR(v,x)
	Definiuje now zmienn v o takim samym typie jak x jednoczenie inicjujc j wartoci x.
**/
#define VAR(v,x)	__typeof(x) v=x

/**
	\def FOREACH(i,c)
	Szybsza w zapisaniu ptla \c for .
**/
#define FOREACH(i,c) for(VAR(i, (c).begin()); i!=(c).end(); ++i)

/**
	\def CONST_FOREACH(i,c)
	Szybsza w zapisaniu ptla \c for , operujca na iteratorach staych lub dynamicznych, w zalenoci od wersji Qt.
**/
#if QT_VERSION >= 0x030200
#define CONST_FOREACH(i,c) for(VAR(i, (c).constBegin()), _kadu__##i##__end = (c).constEnd(); i!=_kadu__##i##__end; ++i)
#else
#define CONST_FOREACH(i,c) for(VAR(i, (c).begin()), _kadu__##i##__end = (c).end(); i!=_kadu__##i##__end; ++i)
#endif

#endif
