#ifndef NOTIFY_H
#define NOTIFY_H
#include <qobject.h>
#include <qstring.h>
#include <qmap.h>
#include <qvariant.h>
#include <qpair.h>
#include <qvaluelist.h>
#include <qstringlist.h>

#include "gadu.h"
#include "notify_slots.h"
#include "userlist.h"

class Notify : public QObject
{
	Q_OBJECT
	private:
		class Notifier
		{
			public:
			Notifier();
			Notifier(QObject *notifier, const QMap<QString, QString> &notifierSlots);

			QObject *notifier; //obiekt "nasuchujcy" na zdarzenia
			QMap<QString, QPair<QString, bool> > notifierSlots; //nazwa sygnau("NewMessage") -> para<slot (SLOT(mjSlotMessage)), czy poczony>
		};
		QMap<QString, Notifier> notifiers; //nazwa powiadamiacza("Hints") -> obiekt powiadomienia
		QMap<QString, QString> notifySignals; //nazwa sygnau("NewMessage") -> sygna (SIGNAL(newMessage(...)))
		QStringList eventNames;

		/*
		 * dodaje kolumn checkboksw w konfiguracji,
		 * na podstawie notifierSlots decyduje czy doda checkboksa aktywnego czy nie
		 */
		void addConfigColumn(const QString &name, const QMap<QString, QString> &notifierSlots);

		/* usuwa kolumn checkboksw z konfiguracji */
		void removeConfigColumn(const QString &name, const QMap<QString, QPair<QString, bool> > &notifierSlots);

	private slots:
		/* pomocniczy slot */
		void newChatSlot(UinsList senders, const QString& msg, time_t time);
		/* pomocniczy slot */
		void probablyNewMessage(UinsList senders, const QString& msg, time_t time, bool &);

		/*
		 * czy odpowiedni sygna z notifierName (np.: Window, Hint, Sound)
		 * ze slotem slotName (np.: NewChat, ConnError)
		 */
		void connectSlot(const QString &notifierName, const QString &slotName);

		/*
		 * rozcza odpowiedni sygna z notifierName (np.: Window, Hint, Sound)
		 * od slotu slotName (np.: NewChat, ConnError)
		 */
		void disconnectSlot(const QString &notifierName, const QString &slotName);

	public:
		Notify(QObject *parent=0, const char *name=0);
		~Notify();

		/*
		 * rejestruje obiekt, ktry chce otrzymywa informacje o zdarzeniach
		 * name bdzie uyte w oknie konfiguracji (naley podawa angielsk nazw)
		 * slots to mapa odwzorujca nazwy sygnaw na sloty w obiekcie notifier
		 * nazwy sygnaw:
			"NewChat" - nowa rozmowa
			"NewMessage" - nowa wiadomo
			"ConnError" - bd poczenia
			"StatusChanged" - zmiana statusu oglna
			"toAvailable" - uytkownik zmieni status na "Dostpny"
			"toBusy" - uytkownik zmieni status na "Zaraz wracam"
			"toInvisible" - uytkownik zmieni status na "Ukryty"
			"toNotAvailable" - uytkownik zmieni status na "Niewidoczny lub Niedostpny"
			"Message" - inna wiadomo
		 *
		 * nie trzeba definiowa wszystkich slotw
		 */
		void registerNotifier(const QString &name, QObject *notifier,
							const QMap<QString, QString> &notifierSlots);
		/*
		 * wyrejestrowuje obiekt
		 */
		void unregisterNotifier(const QString &name);

		QStringList notifiersList() const;
	
	public slots:
		/*
		 * aktualizuje wszystkie poczenia
		 * wywoywany po zmianie konfiguracji dotyczcej obsugi
		 */
		void updateConnections();

		/*
		 * emituje sygna message
		 *  jeeli to==QString::null, to wysya do wszystkich, ktrzy s zaznaczeni w konfiguracji
		 *  jeeli to!=QString::null, to wysya tylko do jednego
		 * Notifier decyduje, ktrych argumentw uy
		 */
		void emitMessage(const QString &from, const QString &to, const QString &message=QString(), const QMap<QString, QVariant> *parameters=NULL, const UserListElement *ule=NULL);

		/* obsuga zmian statusw */
		void userStatusChanged(const UserListElement &, const UserStatus &oldStatus, bool onConnection);

	signals:
	//UWAGA: razem ze zmianami nazw/parametrw tych sygnaw naley aktualizowa wpisy w konstruktorze Notify

		/* nowa rozmowa */
		void newChat(const UinsList &senders, const QString& msg, time_t time);

		/* nowa wiadomo w oknie chat */
		void newMessage(const UinsList &senders, const QString& msg, time_t time, bool &grab);

		/* bd poczenia */
		void connectionError(const QString &message);

		/* uytkownik zmieni status */
		void userStatusChanged(const UserListElement &, const UserStatus &oldStatus);

		/* uytkownik zmieni status na "Dostpny" */
		void userChangedStatusToAvailable(const UserListElement &);

		/* uytkownik zmieni status na "Zaraz wracam" */
		void userChangedStatusToBusy(const UserListElement &);

		/* uytkownik zmieni status na "Niewidoczny" */
		void userChangedStatusToInvisible(const UserListElement &);

		/* uytkownik zmieni status na "Niewidoczny" lub "Niedostpny" */
		void userChangedStatusToNotAvailable(const UserListElement &);

		/* inna informacja do powiadomienia */
		void message(const QString &from, const QString &msg, const QMap<QString, QVariant> *parameters, const UserListElement *ule);

	signals:
		/* do uytku wewntrznego */
		void privateMessage(const QString &from, const QString &message, const QMap<QString, QVariant> *parameters, const UserListElement *ule);
};

extern Notify *notify;
extern NotifySlots *notify_slots;

#endif
