#ifndef KADU_USERBOX_H
#define KADU_USERBOX_H

#include <qlistbox.h>
#include <qpainter.h>
#include <qpopupmenu.h>
#include <qstringlist.h>
#include <qtooltip.h>

#include "misc.h"
#include "userlist.h"

class QFontMetrics;
class UserBoxSlots;
/**
	Klasa reprezentujca kontakt wizualnie na licie kontaktw. Opisuje ona ikon kontaktu,
	jego wywitlan nazw oraz ewentualny opis.
	\class KaduListBoxPixmap
	\brief Klasa reprezentujca kontakt wizualnie.
**/
class KaduListBoxPixmap : public QListBoxItem
{
	public:
		/**
			\fn KaduListBoxPixmap(const QPixmap &pix, const QString &text)
			Konstruktor dodajcy kontakt do listy z podan ikon oraz nazw.
			\param pix ikona statusu.
			\param test wywietlana nazwa kontaktu.
		**/
		KaduListBoxPixmap(const QPixmap &pix, const QString &text);

		/**
			\fn KaduListBoxPixmap(const QPixmap &pix, const QString &text, const QString &descr, bool bold)
			Konstruktor dodajcy kontakt do listy z podan ikon, nazw, oraz opisem.
			\param pix ikona statusu.
			\param test wywietlana nazwa kontaktu.
			\param descr wywietlany opis pod nazw kontaktu.
			\param bold warto logiczna informujca o tym, czy nazwa kontaktu
				ma by wywietlana pogrubion czcionk.
		**/
		KaduListBoxPixmap(const QPixmap &pix, const QString &text, const QString &descr, bool bold);

		/**
			\fn const QString &description() const
			Zwraca wywietlany opis kontaktu.
		**/
		const QString &description() const { return descr; }

		/**
			\fn bool isBold() const
			Zwraca informacj, czy nazwa kontaktu jest wywietlana pogrubion czcionk, czy nie.
		**/
		bool isBold() const { return bold; }

		/**
			\fn int height(const QListBox *lb) const
			Zwraca wysoko elementu reprezentujcego wywietlany kontakt w pikselach.
			\param lb obiekt reprezentujcy wywietlany kontakt.
		**/
		int height(const QListBox *lb) const;

		/**
			\fn int width(const QListBox *lb) const
			Zwraca szeroko elementu reprezentujcego wywietlany kontakt w pikselach.
			\param lb obiekt reprezentujcy wywietlany kontakt.
		**/
		int width(const QListBox *lb) const;

	protected:
		/**
			\fn int lineHeight(const QListBox *lb) const
			Zwraca wysoko samej czcionki dla elementu reprezentujcego wywietlany kontakt w pikselach.
			\param lb obiekt reprezentujcy wywietlany kontakt.
		**/
		int lineHeight(const QListBox *lb) const;

		/**
			\fn void paint(QPainter *painter)
			Rysuje wywietlany kontakt na licie.
			\param painter wskanik do obiektu rysujcego.
		**/
		void paint(QPainter *painter);

		/**
			\fn void setDescription(const QString &d)
			Ustawia wywietlany opis na podany.
			\param d opis do wywietlenia.
		**/
		void setDescription(const QString &d) { descr = d; }

		/**
			\fn void setBold(bool b)
			Ustawia stan pogrubienia czcionki wywietlanego kontaktu.
			\param b warto logiczna okrelajca, czy czcionka ma by pogrubiona czy nie.
		**/
		void setBold(bool b) { bold = b; }

		/**
			\fn void calculateSize(const QString &text, int width, QStringList &out, int &height) const
			Oblicza odpowiedni wysoko elementu listy kontaktw tak, aby pomieci
			opis podzielony na niezbdn liczb linii.
			\param[in] text wywietlany opis.
			\param[in] width szeroko kolumny listy kontaktw.
			\param[out] out lista acuchw znakw, zawierajca kolejne linie opisu powstae
				w wyniku podzielenia opisu tak, aby zmieci si w danej szerokoci kolumny.
			\param[out] height wysoko elementu listy kontaktw potrzebna aby pomieci opis
				podzielony w liniach.
		**/
		void calculateSize(const QString &text, int width, QStringList &out, int &height) const;

		/**
			\fn void changeText(const QString &text)
			Zmienia nazw wywietlan dla kontaktu na licie na podan.
			\param text nowa nazwa do wywietlenia.
		**/
		void changeText(const QString &text);
		friend class UserBox;
		friend class UserBoxSlots;

		/*funkcje wprowadzone eby zaoszczdzi na odwoaniach do pliku konfiguracyjnego*/
		/**
			\fn static void setFont(const QFont &f)
			Ustawia czcionk dla tego elementu.
			\param f czcionka
		**/
		static void setFont(const QFont &f);

		/**
			\fn static void setMyUIN(UinType u)
			Ustawia UIN lokalnego uytkownika Kadu dla tego elementu, aby byo wiadomo,
			e ten element jego reprezentuje (poniewa do pobierania informacji o statusie i opisie
			uywane s rne metody dla kontaktu i lokalnego uytkownika).
			\param u numer UIN.
		**/
		static void setMyUIN(UinType u);

		/**
			\fn static void setShowDesc(bool sd)
			Ustawia stan wywietlania opisw na licie kontaktw.
			\param sd warto logiczna informujca o tym, czy opisy maj by wywietlane na licie kontatw.
		**/
		static void setShowDesc(bool sd);

		/**
			\fn static void setAlignTop(bool at)
			Ustawia stan wyrwnywania do grnego brzegu elementu.
			\param at warto logiczna informujca o tym, czy wywietlana nazwa kontaktu ma by wyrwnywana
				do grnej krawdzi elementu.
		**/
		static void setAlignTop(bool at);

		/**
			\fn static void setShowMultilineDesc(bool m)
			Ustawia stan wywietlania opisw wieloliniowych.
			\param m warto logiczna informujca o tym, czy opis tego elementu moe by wywietlany
				w wielu liniach, jeli zawiera znaki nowej linii.
		**/
		static void setShowMultilineDesc(bool m);

		/**
			\fn static void setMultiColumn(bool m)
			Ustawia stan wywietlania listy kontaktw w wielu kolumnach.
			\param m warto logiczna informujca o tym, czy lista ma by wywietlana w wielu kolumnach.
		**/
		static void setMultiColumn(bool m);

		/**
			\fn static void setMultiColumnWidth(int w)
			Ustawia szeroko jednej kolumny dla wywietlania listy w trybie wielu kolumn.
			\param w szeroko kolumny w pikselach.
		**/
		static void setMultiColumnWidth(int w);

		/**
			\fn static void setDescriptionColor(const QColor &col)
			Ustawia kolor wywietlanego opisu.
			\param col kolor
		**/
		static void setDescriptionColor(const QColor &col);

	private:
		QPixmap pm;
		QString descr;
		bool bold;
		static QFontMetrics *descriptionFontMetrics;

		static UinType myUIN;
		static bool ShowDesc;
		static bool AlignUserboxIconsTop;
		static bool ShowMultilineDesc;
		static bool MultiColumn;
		static int  MultiColumnWidth;
		static QColor descColor;
		
		mutable QString buf_text;
		mutable int buf_width;
		mutable QStringList buf_out;
		mutable int buf_height;
};

/**
	Klasa reprezentujca menu podrczne listy kontaktw - czyli to, ktre dostajemy klikajc
	na ktrymkolwiek kontakcie prawym przyciskiem myszy.
	\class UserBoxMenu
	\brief Menu podrczne listy kontaktw.
**/
class UserBoxMenu : public QPopupMenu
{
	Q_OBJECT

	private:
		QValueList<QPair<QString, QString> > iconNames;
	private slots:
		void restoreLook();

	public:
		/**
			\fn UserBoxMenu(QWidget *parent=0, const char* name=0)
			Standardowy konstruktor.
			\param parent wskanik na obiekt kontrolki-rodzica.
			\param name nazwa kontrolki.
		**/
		UserBoxMenu(QWidget *parent=0, const char* name=0);

		/**
			\fn int getItem(const QString &caption) const
			Zwraca unikaln liczb identyfikujc element menu z podanym napisem.
			\param caption napis elementu.
		**/
		int getItem(const QString &caption) const;
	
	public slots:
		/**
			\fn void show(QListBoxItem *item)
			Pokazuje menu dla podanego kontaktu.
			\param item element wywietlanej listy kontaktw reprezentujcy rzdany kontakt.
		**/
		void show(QListBoxItem *item);

		/**
			\fn int addItem(const QString &text, const QObject* receiver, const char* member, const QKeySequence accel= 0, int id= -1)
			Dodaje nowy element do menu.
			\param text napis dla nowego elementu.
			\param receiver obiekt odbierajcy sygna wybrania elementu z menu.
			\param member SLOT obiektu \a receiver ktry zostanie wykonany po wybraniu elementu z menu.
			\param accel skrt klawiaturowy dla tego elementu. Domylnie brak.
			\param id Unikatowa liczba identyfikujca nowy element. Domylnie pierwsza wolna.
		**/
		int addItem(const QString &text, const QObject* receiver, const char* member, const QKeySequence accel= 0, int id= -1);

		/**
			\fn int addItem(const QString &iconname, const QString &text, const QObject* receiver, const char* member, const QKeySequence accel= 0, int id= -1)
			Dodaje nowy element do menu.
			\param iconname nazwa ikony z zestawu lub sciezka do pliku
			\param text napis dla nowego elementu.
			\param receiver obiekt odbierajcy sygna wybrania elementu z menu.
			\param member SLOT obiektu \a receiver ktry zostanie wykonany po wybraniu elementu z menu.
			\param accel skrt klawiaturowy dla tego elementu. Domylnie brak.
			\param id Unikatowa liczba identyfikujca nowy element. Domylnie pierwsza wolna.
		**/
		int addItem(const QString &iconname, const QString &text, const QObject* receiver, const char* member, const QKeySequence accel= 0, int id= -1);

		/**
			\fn int addItemAtPos(int index,const QString &iconname, const QString &text, const QObject* receiver, const char* member, const QKeySequence accel= 0, int id= -1)
			Dodaje nowy element do menu.
			\param index pozycja (liczc od 0) na ktrej znale ma si nowy element.
			\param iconname nazwa ikony z zestawu lub sciezka do pliku
			\param text napis dla nowego elementu.
			\param receiver obiekt odbierajcy sygna wybrania elementu z menu.
			\param member SLOT obiektu \a receiver ktry zostanie wykonany po wybraniu elementu z menu.
			\param accel skrt klawiaturowy dla tego elementu. Domylnie brak.
			\param id Unikatowa liczba identyfikujca nowy element. Domylnie pierwsza wolna.
		**/
		int addItemAtPos(int index,const QString &iconname, const QString &text, const QObject* receiver, const char* member, const QKeySequence accel= 0, int id= -1);
		
		/**
			\fn void refreshIcons()
			Przeadowuje wszystkie ikony w tym menu dodane przy pomocy powyszych funkcji 
		**/
		void refreshIcons();
	signals:
		/**
			\fn void popup()
			Sygna emitowany, gdy menu jest wywoywane.
		**/
		void popup();
};


/**
	Klasa reprezentujca list kontaktw wraz z ikonkami stanw.
	\class UserBox
	\brief Wywietlana lista kontaktw.
**/
class UserBox : public QListBox , QToolTip
{
	Q_OBJECT

	private:
		static QValueList<UserBox*> UserBoxes;
		static UserBoxSlots *userboxslots;
		// Pixmaps
		static QPixmap* OnlineMobilePixmap;
		static QPixmap* OnlineDescMobilePixmap;
		static QPixmap* BusyMobilePixmap;
		static QPixmap* BusyDescMobilePixmap;
		static QPixmap* OfflineMobilePixmap;
		static QPixmap* OfflineDescMobilePixmap;		
		//
		QStringList Users;
		void sortUsersByAltNick(QStringList &);
			
	protected:
		/**
			\fn virtual void maybeTip(const QPoint& c)
			Wywietla dymek w danym punkcie z informacj o kontakcie znajdujcym si w tym punkcie
			wzgldem lewego grnego rogu listy kontaktw.
			\param c wsprzdne punktu
		**/
		virtual void maybeTip(const QPoint&);

		/**
			\fn virtual void mousePressEvent(QMouseEvent *e)
			Wcinito ktry z przyciskw myszki na licie kontaktw.
			\param e wskanik obiektu opisujcego to zdarzenie.
		**/
		virtual void mousePressEvent(QMouseEvent *e);

		/**
			\fn virtual void mouseMoveEvent(QMouseEvent* e)
			Poruszono myszk nad list kontaktw.
			\param e wskanik obiektu opisujcego to zdarzenie.
		**/
		virtual void mouseMoveEvent(QMouseEvent* e);

		/**
			\fn virtual void keyPressEvent(QKeyEvent *e)
			Wcinito ktry z klawiszw w aktywnej licie kontaktw.
			\param e wskanik obiektu opisujcego to zdarzenie.
		**/
		virtual void keyPressEvent(QKeyEvent *e);

		/**
			\fn virtual void resizeEvent(QResizeEvent *)
			Lista kontaktw zmienia swj rozmiar.
			\param e wskanik obiektu opisujcego to zdarzenie.
		**/
		virtual void resizeEvent(QResizeEvent *);

	public:
		/**
			\fn UserBox(QWidget* parent=0, const char* name=0, WFlags f=0)
			Standardowy konstruktor tworzcy list kontaktw.
			\param parent rodzic kontrolki. Domylnie 0.
			\param name nazwa kontrolki. Domylnie 0.
			\param f flagi kontrolki. Domylnie 0.
		**/
		UserBox(QWidget* parent=0,const char* name=0,WFlags f=0);
		~UserBox();

		/**
			\var static UserBoxMenu *userboxmenu
			Wskanik do menu kontekstowego listy kontaktw.
		**/
		static UserBoxMenu *userboxmenu;

		/**
			\fn bool containsAltNick(const QString &altnick) const
			Funkcja sprawdza czy dany altnick, znajduje si na liscie w UserBox.
		**/
		bool containsAltNick(const QString &altnick) const;

		/**
			\fn static void initModule()
			Inicjalizuje zmienne niezbdne do dziaania UserBox. Funkcja ta jest
			wywoywana przy starcie Kadu przez rdze Kadu.
		**/
		static void initModule();
		static void closeModule();
		/**
			\fn UinsList getSelectedUins() const
			Funkcja zwraca list zaznaczonych uytkownikw.
			\return UinsList z zaznaczonymi uytkownikami posiadajcymi numer GG
		**/
		UinsList getSelectedUins() const;

		/**
			\fn UserList getSelectedUsers() const
			Funkcja zwraca list zaznaczonych uzytkownikw.
			\return UserList z zaznaczonymi uytkownikami.
		**/
		UserList getSelectedUsers() const;

		/**
			\fn QStringList getSelectedAltNicks() const
			Funkcja zwraca list zaznaczonych uzytkownikw.
			\return QStringList z pseudonimami wywietlanymi (altNick'ami).
		**/
		QStringList getSelectedAltNicks() const;

		/**
			\fn static UserBox* getActiveUserBox()
			Funkcja znajdujaca aktywny UserBox.
			\return wskanik do aktywnego UserBox'a, jeli taki nie istnieje zwracana jest warto NULL.
		**/
		static UserBox* getActiveUserBox();

	public slots:
		/**
			\fn void refresh()
			Odwierza list (rysuje od nowa).
		**/
		void refresh();

		/**
			\fn void showHideInactive()
			Przecza tryb pokazywania kontaktw niedostpnych.
		**/
		void showHideInactive();

		/**
			\fn void showHideDescriptions()
			Przecza tryb pokazywania opisw kontaktw.
		**/
		void showHideDescriptions();

		/**
			\fn virtual void clear()
			Czyli list.
		**/
		virtual void clear() { QListBox::clear(); };

		/**
			\fn void clearUsers()
			Funkcja czyci ca list uytkownikw w UserBox.
		**/
		void clearUsers() { Users.clear(); };

		/**
			\fn void addUser(const QString &altnick)
			Funkcja dodaje uytkownika o podanym pseudonimie wywietlanym (altNicku) do listy w UserBox.
		**/
		void addUser(const QString &altnick);

		/**
			\fn void removeUser(const QString &altnick)
			Funkcja usuwa uytkownika o podanym pseudonimie wywietlanym (altNick) z listy w UserBox.
		**/
		void removeUser(const QString &altnick);

		/**
			\fn void renameUser(const QString &oldaltnick, const QString &newaltnick)
			Funkcja zmienia pseudonim wywietlany z podanego na inny w licie UserBox.
		**/
		void renameUser(const QString &oldaltnick, const QString &newaltnick);

		/**
			\fn void changeAllToInactive()
			Ustawia statusy dla wszystkich kontaktw na licie na niedostpny.
			\note Chodzi tu tylko o te widoczne statusy, rzeczywiste statusy kontaktw nadal
			bd znanie poprawnie w instancjach UserListElement.
		**/
		void changeAllToInactive();

		// Functions below works on all created userboxes
		/**
			\fn static void all_refresh()
			Wykonuje metod UserBox::refresh() na wszystkich istniejcych instancjach UserBox.
		**/
		static void all_refresh();

		/**
			\fn static void all_removeUser(const QString &altnick)
			Funkcja usuwa uytkownika o podanym pseudonimie wywietlanym (altNick) z wszystkich
			instancji UserBox.
		**/
		static void all_removeUser(const QString &altnick);

		/**
			\fn static void all_changeAllToInactive()
			Wykonuje metod UserBox::changeAllToInactive() na wszystkich istniejcych instancjach
			UserBox.
		**/
		static void all_changeAllToInactive();

		/**
			\fn static void all_renameUser(const QString &oldaltnick, const QString &newaltnick)
			Funkcja zmienia pseudonim wywietlany z podanego na inny we wszystkich instancjach UserBox.
		**/
		static void all_renameUser(const QString &oldaltnick, const QString &newaltnick);

	signals:
		/**
			\fn void doubleClicked(const QString &text)
			Sygna jest emitowany, gdy na ktrym z kontaktw kliknito dwukrotnie.
			\param text pseudonim wywietlany (altNick) kontaktu na ktrym dwu-kilknito.
			\warning Uywaj tego sygnau zamiast QListBox::doubleClicked(QListBoxItem *) !!!
			Tamten ze wzgldu na odwieanie listy w jednym ze slotw podczonych
			do tego sygnau czasami przekazuje wskanik do elementu, ktry ju NIE ISTNIEJE.
		**/
		void doubleClicked(const QString &text);

		/**
			\fn void returnPressed(const QString &text)
			Sygna jest emitowany, gdy wcinito klawisz enter na wybranym kontakcie.
			\param text pseudonim wywietlany (altNick) kontaktu na ktrym wcinito enter.
			\warning Uywaj tego sygnau zamiast QListBox::returnPressed(QListBoxItem *) !!!
			Tamten ze wzgldu na odwieanie listy w jednym ze slotw podczonych
			do tego sygnau czasami przekazuje wskanik do elementu, ktry ju NIE ISTNIEJE.
		**/
		void returnPressed(const QString &text);
	private slots:
		void doubleClickedSlot(QListBoxItem *item);
		void returnPressedSlot(QListBoxItem *item);
};

/**
	Klasa ta pozwala na podgld wprowadzanych zmian konfiguracyjnych wobec UserBox
	w oknie konfiguracji, zanim zaaplikuje si je do Kadu.
	\class UserBoxSlots
	\brief Obsuga UserBox w konfiguracji.
**/
class UserBoxSlots : public QObject
{
	Q_OBJECT
	public slots:
		/**
			\fn void onCreateConfigDialog()
			Obsuguje sekcj UserBox podczas otwierania okna konfiguracji.
		**/
		void onCreateConfigDialog();

		/**
			\fn void onDestroyConfigDialog()
			Obsuguje sekcj UserBox podczas zamykania okna konfiguracji.
		**/
		void onDestroyConfigDialog();

		/**
			\fn void chooseColor(const char *name, const QColor& color)
			Odwierza podgld wybranego koloru.
			\param name nazwa elementu, dla ktrego wybrano kolor.
				\arg \c userbox_bg_color oznacza kolor ta.
				\arg \c userbox_font_color oznacz kolor czcionki.
			\param color wybrany kolor.
		**/
		void chooseColor(const char *name, const QColor& color);

		/**
			\fn void chooseFont(const char *name, const QFont& font)
			Odwierza podgld wybranej czcionki.
			\param name nazwa elementu, dla ktrego wybrano czcionk.
				\arg \c userbox_font_box oznacza ogln czcionk listy kontaktw.
			\param font wybrana czcionka.
		**/
		void chooseFont(const char *name, const QFont& font);

		/**
			\fn void onMultiColumnUserbox(bool toggled)
			Wcza bd wycza kontrolk szerokoci kolumny listy kontaktw, w zalenoci od podanego argumentu.
			\param togglet warto logiczna informujca, czy kontrolka ma by wczona czy wyczona.
		**/
		void onMultiColumnUserbox(bool toggled);

		/**
			\fn void updatePreview()
			Odwierza podgld wszystkich elementw UserBox'a.
		**/
		void updatePreview();
};

#endif
